package com.sw.mes.itemclean.clean.controller;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynmaicQueryWrapper;
import com.sw.mes.core.utils.RequestPage;
import com.sw.mes.itemclean.clean.dto.MaterialBrokenDetailDto;
import com.sw.mes.itemclean.clean.domain.MaterialBrokenDetail;
import com.sw.mes.itemclean.clean.mapper.MaterialBrokenDetailMapper;
import com.sw.mes.itemclean.clean.service.MaterialBrokenDetailService;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;


@RestController
@RequestMapping("/clean/materialBrokenDetail")
@Slf4j
@Api(tags = "破碎明细(对应料箱明细)")
public class MaterialBrokenDetailController {
    @Autowired
    ModelMapper modelMapper;

    @Autowired
    private MaterialBrokenDetailService materialBrokenDetailService;

    @Autowired
    private MaterialBrokenDetailMapper materialBrokenDetailMapper;

    @ApiOperation(value = "分页查询列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
            @ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
    })
    @GetMapping("/listByPage")
    public Result<ResponsePage<MaterialBrokenDetailDto>> listByPage(ServletRequest request, @ApiIgnore RequestPage requestPage) {
        //分页查询
        QueryWrapper<MaterialBrokenDetail> queryWrapper =  DynmaicQueryWrapper.searchFilter(request, MaterialBrokenDetail.class);
        Page<MaterialBrokenDetail> listPage = materialBrokenDetailService.page(new Page<>(requestPage.getPage(), requestPage.getRows()), queryWrapper);
        List<MaterialBrokenDetailDto> dtos = listPage.getRecords().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
        ResponsePage responsePage = new ResponsePage(dtos, listPage.getTotal());
        return ResultGenerator.success(responsePage);
    }

    @ApiOperation(value = "获取所有列表")
    @GetMapping("/listAll")
    public Result<List<MaterialBrokenDetailDto>> listAll(ServletRequest request) {
        QueryWrapper<MaterialBrokenDetail> queryWrapper =  DynmaicQueryWrapper.searchFilter(request, MaterialBrokenDetail.class);
        List<MaterialBrokenDetail> materialBrokenDetails = materialBrokenDetailService.list(queryWrapper);
        List<MaterialBrokenDetailDto> materialBrokenDetailDtos = materialBrokenDetails.stream()
                .map(materialBrokenDetail -> toLazyDto(materialBrokenDetail))
                .collect(Collectors.toList());
        return ResultGenerator.success(materialBrokenDetailDtos);
    }

    @ApiOperation(value = "详细信息")
    @GetMapping("/{id}")
    public Result<MaterialBrokenDetailDto> get(@PathVariable @ApiParam(value="设备类型id") Integer id) {
        MaterialBrokenDetail materialBrokenDetail = materialBrokenDetailService.getById(id);
        if (null == materialBrokenDetail)
            return ResultGenerator.fail("该对象不存在");
        return ResultGenerator.success(toEagerDto(materialBrokenDetail));
    }

    @ApiOperation(value = "创建")
    @PostMapping("/create")
    public Result create(@RequestBody @Valid @ApiParam(value = "Dto对象") MaterialBrokenDetailDto materialBrokenDetailDto, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        }
        MaterialBrokenDetail materialBrokenDetail = fromDto(materialBrokenDetailDto);
        materialBrokenDetailService.saveOrUpdate(materialBrokenDetail);
        return ResultGenerator.success();
    }

    @ApiOperation(value = "更新")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "Dto对象") MaterialBrokenDetailDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors())
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        if (dto.getId() == null)
            return ResultGenerator.fail("id不能为空");
        MaterialBrokenDetail entity = materialBrokenDetailService.getById(dto.getId());
        if (entity == null)
            return ResultGenerator.fail("原信息不存在");
        try {
            MaterialBrokenDetail entityfromFto = fromDto(dto);
            BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
            materialBrokenDetailService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

    @ApiOperation(value = "删除")
    @PostMapping("/delete")
    public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
        for (Integer id : ids) {
            materialBrokenDetailMapper.deleteById(id);
        }
        return ResultGenerator.success();
    }

    public MaterialBrokenDetailDto toLazyDto(MaterialBrokenDetail entity) {
        return modelMapper.map(entity, MaterialBrokenDetailDto.class);
    }

    public MaterialBrokenDetailDto toEagerDto(MaterialBrokenDetail entity) {
        MaterialBrokenDetailDto dto = modelMapper.map(entity, MaterialBrokenDetailDto.class);
        return dto;
    }

    public MaterialBrokenDetail fromDto(MaterialBrokenDetailDto dto) {
        return modelMapper.map(dto, MaterialBrokenDetail.class);
    }
}
