package com.sw.mes.itemclean.clean.controller;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynmaicQueryWrapper;
import com.sw.mes.core.utils.RequestPage;
import com.sw.mes.itemclean.clean.dto.MaterialLandingMaterialSignatureDto;
import com.sw.mes.itemclean.clean.domain.MaterialLandingMaterialSignature;
import com.sw.mes.itemclean.clean.mapper.MaterialLandingMaterialSignatureMapper;
import com.sw.mes.itemclean.clean.service.GenerateBusinessNoService;
import com.sw.mes.itemclean.clean.service.MaterialLandingMaterialSignatureService;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;

@RestController
@RequestMapping("/clean/materialLandingMaterialSignature")
@Slf4j
@Api(tags = "落地料打签")
public class MaterialLandingMaterialSignatureController {
    @Autowired
    ModelMapper modelMapper;

    @Autowired
    private MaterialLandingMaterialSignatureService materialLandingMaterialSignatureService;

    @Autowired
    GenerateBusinessNoService generateBusinessNoService;

    @Autowired
    private MaterialLandingMaterialSignatureMapper materialLandingMaterialSignatureMapper;

    @ApiOperation(value = "分页查询列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
            @ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
    })
    @GetMapping("/listByPage")
    public Result<ResponsePage<MaterialLandingMaterialSignatureDto>> listByPage(ServletRequest request, @ApiIgnore RequestPage requestPage) {
        //分页查询
        QueryWrapper<MaterialLandingMaterialSignature> queryWrapper =  DynmaicQueryWrapper.searchFilter(request, MaterialLandingMaterialSignature.class);
        Page<MaterialLandingMaterialSignature> listPage = materialLandingMaterialSignatureService.page(new Page<>(requestPage.getPage(), requestPage.getRows()), queryWrapper);
        List<MaterialLandingMaterialSignatureDto> dtos = listPage.getRecords().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
        ResponsePage responsePage = new ResponsePage(dtos, listPage.getTotal());
        return ResultGenerator.success(responsePage);
    }

    @ApiOperation(value = "获取所有列表")
    @GetMapping("/listAll")
    public Result<List<MaterialLandingMaterialSignatureDto>> listAll(ServletRequest request) {
        QueryWrapper<MaterialLandingMaterialSignature> queryWrapper =  DynmaicQueryWrapper.searchFilter(request, MaterialLandingMaterialSignature.class);
        List<MaterialLandingMaterialSignature> materialLandingMaterialSignatures = materialLandingMaterialSignatureService.list(queryWrapper);
        List<MaterialLandingMaterialSignatureDto> materialLandingMaterialSignatureDtos = materialLandingMaterialSignatures.stream()
                .map(materialLandingMaterialSignature -> toLazyDto(materialLandingMaterialSignature))
                .collect(Collectors.toList());
        return ResultGenerator.success(materialLandingMaterialSignatureDtos);
    }

    @ApiOperation(value = "详细信息")
    @GetMapping("/{id}")
    public Result<MaterialLandingMaterialSignatureDto> get(@PathVariable @ApiParam(value="设备类型id") Integer id) {
        MaterialLandingMaterialSignature materialLandingMaterialSignature = materialLandingMaterialSignatureService.getById(id);
        if (null == materialLandingMaterialSignature)
            return ResultGenerator.fail("该对象不存在");
        return ResultGenerator.success(toEagerDto(materialLandingMaterialSignature));
    }

    @ApiOperation(value = "创建")
    @PostMapping("/create")
    public Result create(@RequestBody @Valid @ApiParam(value = "Dto对象") MaterialLandingMaterialSignatureDto materialLandingMaterialSignatureDto, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        }
        MaterialLandingMaterialSignature materialLandingMaterialSignature = fromDto(materialLandingMaterialSignatureDto);
        materialLandingMaterialSignatureService.saveOrUpdate(materialLandingMaterialSignature);
        return ResultGenerator.success();
    }

    @ApiOperation(value = "根据输入的物料名称获取物料信息")
    @PostMapping("/findByMaterialCleanName")
    public Result findByMaterialCleanName(@ApiParam(value = "物料名称", type = "query") @RequestParam String name) {
        Result materialCleanNames = materialLandingMaterialSignatureService.findByMaterialCleanName(name);
        return ResultGenerator.success(materialCleanNames);
    }


    @ApiOperation(value = "落地料成箱打签")
    @PostMapping("/landingMaterialSignature")
    public Result landingMaterialSignature(@RequestBody @Valid @ApiParam(value = "Dto对象") MaterialLandingMaterialSignatureDto materialLandingMaterialSignatureDto, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        }
        return materialLandingMaterialSignatureService.landingMaterialSignature(materialLandingMaterialSignatureDto);
    }

    @ApiOperation(value = "更新")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "Dto对象") MaterialLandingMaterialSignatureDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors())
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        if (dto.getId() == null)
            return ResultGenerator.fail("id不能为空");
        MaterialLandingMaterialSignature entity = materialLandingMaterialSignatureService.getById(dto.getId());
        if (entity == null)
            return ResultGenerator.fail("原信息不存在");
        try {
            MaterialLandingMaterialSignature entityfromFto = fromDto(dto);
            BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
            materialLandingMaterialSignatureService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

    @ApiOperation(value = "删除")
    @PostMapping("/delete")
    public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
        for (Integer id : ids) {
            materialLandingMaterialSignatureMapper.deleteById(id);
        }
        return ResultGenerator.success();
    }

    public MaterialLandingMaterialSignatureDto toLazyDto(MaterialLandingMaterialSignature entity) {
        return modelMapper.map(entity, MaterialLandingMaterialSignatureDto.class);
    }

    public MaterialLandingMaterialSignatureDto toEagerDto(MaterialLandingMaterialSignature entity) {
        MaterialLandingMaterialSignatureDto dto = modelMapper.map(entity, MaterialLandingMaterialSignatureDto.class);
        return dto;
    }

    public MaterialLandingMaterialSignature fromDto(MaterialLandingMaterialSignatureDto dto) {
        return modelMapper.map(dto, MaterialLandingMaterialSignature.class);
    }
}
