package com.sw.mes.itemclean.clean.dto;

import com.alibaba.excel.annotation.ExcelIgnore;
import com.alibaba.excel.annotation.ExcelProperty;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.sw.mes.itemclean.clean.utils.LocalDateStringConverter;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Getter;
import lombok.Setter;

import java.io.Serializable;
import java.math.BigDecimal;
import java.time.LocalDateTime;

/**
 * <p>
 * 清洗
 * </p>
 *
 * @author liuzemin
 * @since 2022-11-10
 */
@Getter
@Setter
@ApiModel(value = "MaterialCleaning对象", description = "清洗")
public class MaterialCleaningDto implements Serializable {

    private static final long serialVersionUID = 1L;

    @ApiModelProperty("主键")
    @ExcelIgnore
    private Integer id;

    @ApiModelProperty("重量（上料总计）")
    @ExcelProperty(value = "上料重量", index = 7)
    private BigDecimal weight;

    @ApiModelProperty("物料编号")
    @ExcelProperty(value = "物料编号", index = 3)
    private String materialCode;

    @ApiModelProperty("物料名称")
    @ExcelProperty(value = "物料名称", index = 2)
    private String materialName;

    @ApiModelProperty("物料等级")
    @ExcelProperty(value = "等级", index = 5)
    private String materialLevel;

    @ApiModelProperty("物料型号")
    @ExcelProperty(value = "型号", index = 4)
    private String materialModel;

    @ApiModelProperty("电阻档位")
    @ExcelProperty(value = "档位", index = 6)
    private String materialGear;

    @ApiModelProperty("分选pp生产批次")
    @ExcelProperty(value = "PP号", index = 1)
    private String productBatchNo;

    @ApiModelProperty("批次完结 1完结 0未完结")
    @ExcelIgnore
    private Integer batchEnd;

    @ApiModelProperty("设备号")
    @ExcelProperty(value = "设备号", index = 0)
    private String deviceNo;

    @ApiModelProperty("创建人-清洗上料人")
    @ExcelProperty(value = "清洗上料人", index = 10)
    private String createUser;

    @ApiModelProperty("创建时间-清洗上料时间")
    @ExcelProperty(value = "清洗上料时间", converter = LocalDateStringConverter.class, index = 9)
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private LocalDateTime createTime;

    @ApiModelProperty("更新人")
    @ExcelIgnore
    private String updateUser;

    @ApiModelProperty("更新时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @ExcelIgnore
    private LocalDateTime updateTime;

    @ApiModelProperty("花篮载具号")
    @ExcelProperty(value = "花篮号", index = 8)
    private String boxNo;

    @ApiModelProperty("下料人")
    @ExcelProperty(value = "下料重量", index = 13)
    private String downUser;

    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @ApiModelProperty("下料时间")
    @ExcelProperty(value = "清洗下料时间", converter = LocalDateStringConverter.class, index = 12)
    private LocalDateTime downTime;

    @ApiModelProperty("下料重量")
    @ExcelProperty(value = "下料重量", index = 11)
    private BigDecimal downWeight;

    @ApiModelProperty("逻辑删除标记")
    @ExcelIgnore
    private Integer delFlag;

    @ApiModelProperty("是否有新批次")
    @ExcelIgnore
    private Integer isNewBatch; //1:有新批次  。  2:无新批次

    @ApiModelProperty("清洗状态 0 清洗上料 1 清洗下料 2 拆花篮成箱 3 待破碎")
    @ExcelIgnore
    private Integer status;

}
