package com.sw.mes.config.rabbit;

import org.springframework.amqp.core.Binding;
import org.springframework.amqp.core.BindingBuilder;
import org.springframework.amqp.core.DirectExchange;
import org.springframework.amqp.core.Queue;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

/**
 * Broker:它提供一种传输服务,它的角色就是维护一条从生产者到消费者的路线，保证数据能按照指定的方式进行传输,
 * Exchange：消息交换机,它指定消息按什么规则,路由到哪个队列。 Queue:消息的载体,每个消息都会被投到一个或多个队列。
 * <p>
 * Binding:绑定，它的作用就是把exchange和queue按照路由规则绑定起来.
 * <p>
 * Routing Key:路由关键字,
 * <p>
 * exchange根据这个关键字进行消息投递。 vhost:虚拟主机,一个broker里可以有多个vhost，用作不同用户的权限分离。
 * <p>
 * Producer:消息生产者,就是投递消息的程序. Consumer:消息消费者,就是接受消息的程序.
 * Channel:消息通道,在客户端的每个连接里,可建立多个channel.
 */

//@Configuration
public class RabbitConfig {
    public static final String FB_INSPECT_DATA = "fbInspectData";
    public static final String FB_INSPECT_DATA_QUEUE = "fbInspectDataQueue";
    public static final String FB_INSPECT_DATA_EXCHANGE = "fbInspectDataExchange";

    /**
     * 方棒检验数据变动队列
     *
     * @return Queue
     */
    //@Bean(FB_INSPECT_DATA_QUEUE)
    public Queue fbInspectDataQueue() {
        return new Queue(FB_INSPECT_DATA_QUEUE, true);
    }

    //@Bean(FB_INSPECT_DATA_EXCHANGE)
    public DirectExchange fbInspectDataExchange() {
        return new DirectExchange(FB_INSPECT_DATA_EXCHANGE);
    }

    //@Bean
    public Binding fbInspectDataExchangeBinding(@Qualifier(FB_INSPECT_DATA_QUEUE) Queue queue, @Qualifier(FB_INSPECT_DATA_EXCHANGE) DirectExchange directExchange) {
        return BindingBuilder.bind(queue).to(directExchange).with(FB_INSPECT_DATA);
    }
}
