package com.sw.mes.fb.accessory.controller;

import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import com.sw.mes.fb.accessory.dto.FbAccessoryOrderDto;
import com.sw.mes.fb.accessory.entity.FbAccessoryOrder;
import com.sw.mes.fb.accessory.service.FbAccessoryOrderService;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;


/**
* @Author
* @Description：
* @Date: Create in
* @Modified By:
*/
@RestController
@RequestMapping("/fbAccessoryOrder")
@Slf4j
@Api(description = "方棒辅料更换记录接口")
public class FbAccessoryOrderController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private FbAccessoryOrderService fbAccessoryOrderService;

	@ApiOperation(value = "分页查询方棒辅料更换记录列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<FbAccessoryOrderDto>> listByPage(ServletRequest request, @ApiIgnore RequestPage requestPage) {
		Specification<FbAccessoryOrder> specification = DynamicSpecifications.bySearchFilter(request, FbAccessoryOrder.class);
    	Page<FbAccessoryOrder> page = fbAccessoryOrderService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<FbAccessoryOrderDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有方棒辅料更换记录")
	@GetMapping("/listAll")
	public Result<List<FbAccessoryOrderDto>> listAll(ServletRequest request) {
		Specification<FbAccessoryOrder> specification = DynamicSpecifications.bySearchFilter(request, FbAccessoryOrder.class);
		List<FbAccessoryOrder> fbAccessoryOrders = fbAccessoryOrderService.findByExampleSort(specification, null);
		List<FbAccessoryOrderDto> fbAccessoryOrderDtos = fbAccessoryOrders.stream()
					.map(fbAccessoryOrder -> toLazyDto(fbAccessoryOrder))
					.collect(Collectors.toList());
		return ResultGenerator.success(fbAccessoryOrderDtos);
	}

	@ApiOperation(value = "方棒辅料更换记录详细信息")
	@GetMapping("/{id}")
	public Result<FbAccessoryOrderDto> get(@PathVariable @ApiParam(value="方棒辅料更换记录id") Long id) {
		FbAccessoryOrder fbAccessoryOrder = fbAccessoryOrderService.get(id);
		if (null == fbAccessoryOrder)
			return ResultGenerator.fail("该方棒辅料更换记录不存在");
		return ResultGenerator.success(toEagerDto(fbAccessoryOrder));
	}

	@ApiOperation(value = "创建方棒辅料更换记录")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "方棒辅料更换记录对象") FbAccessoryOrderDto fbAccessoryOrderDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		FbAccessoryOrder fbAccessoryOrder = fromDto(fbAccessoryOrderDto);
		fbAccessoryOrderService.saveOrUpdate(fbAccessoryOrder);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "更新方棒辅料更换记录")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "方棒辅料更换记录") FbAccessoryOrderDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors())
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        if (dto.getId() == null)
            return ResultGenerator.fail("id不能为空");
            FbAccessoryOrder entity = fbAccessoryOrderService.get(dto.getId());
        if (entity == null)
            return ResultGenerator.fail("原信息不存在");
        try {
            FbAccessoryOrder entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			fbAccessoryOrderService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除方棒辅料更换记录")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
		for (Integer id : ids) {
				fbAccessoryOrderService.delete(id);
		}
		return ResultGenerator.success();
	}

    @ApiOperation(value = "根据唯一码前查询最新的历史记录")
    @GetMapping("/findByGoodsCodeBefore")
    public Result findByOrder(String goodsCodeBefore){
        FbAccessoryOrder order = fbAccessoryOrderService.findByGoodsCodeBefore(goodsCodeBefore);
        return ResultGenerator.success(toLazyDto(order));
    }

	public FbAccessoryOrderDto toLazyDto(FbAccessoryOrder entity) {
		return modelMapper.map(entity, FbAccessoryOrderDto.class);
	}

	public FbAccessoryOrderDto toEagerDto(FbAccessoryOrder entity) {
		FbAccessoryOrderDto dto = modelMapper.map(entity, FbAccessoryOrderDto.class);
		return dto;
	}

	public FbAccessoryOrder fromDto(FbAccessoryOrderDto dto) {
		return modelMapper.map(dto, FbAccessoryOrder.class);
	}


}
