package com.sw.mes.fb.checkout.controller;

import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import com.sw.mes.fb.checkout.dto.FieldExtensionRecordDto;
import com.sw.mes.fb.checkout.entity.FieldExtensionRecord;
import com.sw.mes.fb.checkout.service.FieldExtensionRecordService;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;


/**
* @Author
* @Description：
* @Date: Create in
* @Modified By:
*/
@RestController
@RequestMapping("/fieldExtensionRecord")
@Slf4j
@Api(description = "扩展字段值记录接口")
public class FieldExtensionRecordController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private FieldExtensionRecordService fieldExtensionRecordService;

	@ApiOperation(value = "分页查询扩展字段值记录列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<FieldExtensionRecordDto>> listByPage(ServletRequest request, @ApiIgnore RequestPage requestPage) {
		Specification<FieldExtensionRecord> specification = DynamicSpecifications.bySearchFilter(request, FieldExtensionRecord.class);
    	Page<FieldExtensionRecord> page = fieldExtensionRecordService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<FieldExtensionRecordDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有扩展字段值记录")
	@GetMapping("/listAll")
	public Result<List<FieldExtensionRecordDto>> listAll(ServletRequest request) {
		Specification<FieldExtensionRecord> specification = DynamicSpecifications.bySearchFilter(request, FieldExtensionRecord.class);
		List<FieldExtensionRecord> fieldExtensionRecords = fieldExtensionRecordService.findByExampleSort(specification, null);
		List<FieldExtensionRecordDto> fieldExtensionRecordDtos = fieldExtensionRecords.stream()
					.map(fieldExtensionRecord -> toLazyDto(fieldExtensionRecord))
					.collect(Collectors.toList());
		return ResultGenerator.success(fieldExtensionRecordDtos);
	}

	@ApiOperation(value = "扩展字段值记录详细信息")
	@GetMapping("/{id}")
	public Result<FieldExtensionRecordDto> get(@PathVariable @ApiParam(value="扩展字段值记录id") Long id) {
		FieldExtensionRecord fieldExtensionRecord = fieldExtensionRecordService.get(id);
		if (null == fieldExtensionRecord)
			return ResultGenerator.fail("该扩展字段值记录不存在");
		return ResultGenerator.success(toEagerDto(fieldExtensionRecord));
	}

	@ApiOperation(value = "创建扩展字段值记录")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "扩展字段值记录对象") FieldExtensionRecordDto fieldExtensionRecordDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		FieldExtensionRecord fieldExtensionRecord = fromDto(fieldExtensionRecordDto);
		fieldExtensionRecordService.saveOrUpdate(fieldExtensionRecord);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "更新扩展字段值记录")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "扩展字段值记录") FieldExtensionRecordDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors())
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        if (dto.getId() == null)
            return ResultGenerator.fail("id不能为空");
            FieldExtensionRecord entity = fieldExtensionRecordService.get(dto.getId());
        if (entity == null)
            return ResultGenerator.fail("原信息不存在");
        try {
            FieldExtensionRecord entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			fieldExtensionRecordService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除扩展字段值记录")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
		for (Integer id : ids) {
				fieldExtensionRecordService.delete(id);
		}
		return ResultGenerator.success();
	}

	public FieldExtensionRecordDto toLazyDto(FieldExtensionRecord entity) {
		return modelMapper.map(entity, FieldExtensionRecordDto.class);
	}

	public FieldExtensionRecordDto toEagerDto(FieldExtensionRecord entity) {
		FieldExtensionRecordDto dto = modelMapper.map(entity, FieldExtensionRecordDto.class);
		return dto;
	}

	public FieldExtensionRecord fromDto(FieldExtensionRecordDto dto) {
		return modelMapper.map(dto, FieldExtensionRecord.class);
	}


}
