package com.sw.mes.fb.checkout.controller;

import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import com.sw.mes.fb.checkout.dto.FieldExtensionShowDto;
import com.sw.mes.fb.checkout.dto.FieldExtensionTableDto;
import com.sw.mes.fb.checkout.entity.FieldExtensionTable;
import com.sw.mes.fb.checkout.service.FieldExtensionTableService;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;


/**
* @Author
* @Description：
* @Date: Create in
* @Modified By:
*/
@RestController
@RequestMapping("/fieldExtensionTable")
@Slf4j
@Api(description = "表字段扩展接口")
public class FieldExtensionTableController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private FieldExtensionTableService fieldExtensionTableService;

	@ApiOperation(value = "分页查询表字段扩展列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<FieldExtensionTableDto>> listByPage(ServletRequest request, @ApiIgnore RequestPage requestPage) {
		Specification<FieldExtensionTable> specification = DynamicSpecifications.bySearchFilter(request, FieldExtensionTable.class);
    	Page<FieldExtensionTable> page = fieldExtensionTableService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<FieldExtensionTableDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有表字段扩展")
	@GetMapping("/listAll")
	public Result<List<FieldExtensionTableDto>> listAll(ServletRequest request) {
		Specification<FieldExtensionTable> specification = DynamicSpecifications.bySearchFilter(request, FieldExtensionTable.class);
		List<FieldExtensionTable> fieldExtensionTables = fieldExtensionTableService.findByExampleSort(specification, null);
		List<FieldExtensionTableDto> fieldExtensionTableDtos = fieldExtensionTables.stream()
					.map(fieldExtensionTable -> toLazyDto(fieldExtensionTable))
					.collect(Collectors.toList());
		return ResultGenerator.success(fieldExtensionTableDtos);
	}

	@ApiOperation(value = "表字段扩展详细信息")
	@GetMapping("/{id}")
	public Result<FieldExtensionTableDto> get(@PathVariable @ApiParam(value="表字段扩展id") Long id) {
		FieldExtensionTable fieldExtensionTable = fieldExtensionTableService.get(id);
		if (null == fieldExtensionTable)
			return ResultGenerator.fail("该表字段扩展不存在");
		return ResultGenerator.success(toEagerDto(fieldExtensionTable));
	}

	@ApiOperation(value = "创建表字段扩展")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "表字段扩展对象") FieldExtensionTableDto fieldExtensionTableDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		FieldExtensionTable fieldExtensionTable = fromDto(fieldExtensionTableDto);
		FieldExtensionTable byEnglishName = fieldExtensionTableService.findByEnglishName(fieldExtensionTableDto.getEnglishName());
		if (null != byEnglishName){
			return ResultGenerator.fail("该表字段扩展英文名称已存在");
		}
		fieldExtensionTableService.saveOrUpdate(fieldExtensionTable);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "更新表字段扩展")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "表字段扩展") FieldExtensionTableDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors())
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        if (dto.getId() == null)
            return ResultGenerator.fail("id不能为空");
            FieldExtensionTable entity = fieldExtensionTableService.get(dto.getId());
        if (entity == null)
            return ResultGenerator.fail("原信息不存在");
        try {
            FieldExtensionTable entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			fieldExtensionTableService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除表字段扩展")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
		for (Integer id : ids) {
				fieldExtensionTableService.delete(id);
		}
		return ResultGenerator.success();
	}

	@ApiOperation(value = "表字段扩展详细信息")
	@GetMapping("/getByTable")
	public Result<List<FieldExtensionShowDto>> getByTable(@RequestParam  String extendTable) {
		List<FieldExtensionShowDto> byExtendTable = fieldExtensionTableService.findByExtendTable(extendTable);
		return ResultGenerator.success(byExtendTable);
	}


	public FieldExtensionTableDto toLazyDto(FieldExtensionTable entity) {
		return modelMapper.map(entity, FieldExtensionTableDto.class);
	}

	public FieldExtensionTableDto toEagerDto(FieldExtensionTable entity) {
		FieldExtensionTableDto dto = modelMapper.map(entity, FieldExtensionTableDto.class);
		return dto;
	}

	public FieldExtensionTable fromDto(FieldExtensionTableDto dto) {
		return modelMapper.map(dto, FieldExtensionTable.class);
	}


}
