package com.sw.mes.inspect.entity;

import lombok.Data;
import org.hibernate.annotations.DynamicInsert;
import org.hibernate.annotations.DynamicUpdate;

import javax.persistence.*;
import java.math.BigDecimal;
import java.util.Date;

/**
 * @Author
 * @Description：
 * @Date: Create in
 * @Modified By:
 */
@Data
@Entity
@Table(name = "yb_inspect_data")
@DynamicUpdate
@DynamicInsert
public class YbInspectData {
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Long id;

    /**
     * 晶段编号(短棒)
     */
    @Column(length = 80)
    private String sectionNo;

    /**
     * 单晶编号(整棒)
     */
    @Column(length = 80)
    private String crystalNo;

    /**
     * 单晶组合
     */
    @Column()
    private String sectionCompose;

    /**
     * 单晶段号
     */
    @Column(length = 10)
    private Integer piecewiseNum;

    /**
     * 检验线编号
     */
    @Column(length = 50)
    private String checkLineNo;

    /**
     * 头尾识别
     */
    @Column(length = 50)
    private String headTailRecognition;

    /**
     * 规格/实际规格
     */
    @Column(length = 40)
    private String productTypeSecond;

    /**
     * 掺杂剂
     */
    @Column(length = 10)
    private String dopant;

    /**
     * 导电型号/型号
     */
    @Column(length = 40)
    private String conductiveType;


    /**
     * 是否外发【0：默认，否；1：是；】(新）
     */
    @Column(length = 3)
    private Integer outGoing;

    /**
     * 产品等级
     */
    @Column(length = 80)
    private String productLevel;


    /**
     * 检验类型
     */
    @Column(length = 50)
    private String checkoutName;

    /**
     * 外观检验返切
     */
    @Column(length = 20)
    private String checkReback;

    /**
     * 复检次数
     */
    @Column(length = 40)
    private String reCheckType;

    /**
     * 评级时间
     */
    @Column(length = 19)
    @Temporal(TemporalType.TIMESTAMP)
    private Date levelTime;

    /**
     * 检验时间
     */
    @Column(length = 19)
    @Temporal(TemporalType.TIMESTAMP)
    private Date createTime;

    /**
     * 检验日期（新）
     */
    @Column(length = 50)
    private String inspectionDate;

    /**
     * 单晶减长长度（损耗)
     */
    @Column(length = 19)
    private BigDecimal reduceCrystalLength;

    /**
     * 圆棒长度(mm)
     */
    @Column(length = 19)
    private BigDecimal actualLength;

    /**
     * 理论重量
     */
    @Column(length = 19)
    private BigDecimal theoryWeight;

    /**
     * 实际称重/实测重量
     */
    @Column(length = 19)
    private BigDecimal actualWeight;

    /**
     * 拼棒总长(mm)
     */
    @Column(length = 8)
    private BigDecimal spellBarLength;


    /**
     * 拼数
     */
    @Column()
    private Integer sticksCount;

    /**
     * 晶段类型
     */
    @Column(length = 20)
    private String sectionType;

    /**
     * 是否返切【0：默认，否；1：是；】（新）
     */
    @Column(length = 3)
    private Integer backCut;

    /**
     * 直径Min
     */
    @Column(length = 19)
    private BigDecimal diameterMin;

    /**
     * 直径Max
     */
    @Column(length = 19)
    private BigDecimal diameterMax;

    /**
     * 等级码（新）
     */
    @Column(length = 100)
    private String levelCode;

    /**
     * 原料损耗（新）
     */
    @Column(length = 19)
    private BigDecimal materialLoss;

    /**
     * 阴阳面孪晶距头位置（新）
     */
    @Column(length = 11)
    private Integer twinCrystalPosition;

    /**
     * 阴阳面孪晶长度（新）
     */
    @Column(length = 11)
    private Integer twinCrystalLength;

    /**
     * 带状孪晶距头位置（新）
     */
    @Column(length = 11)
    private Integer twinBandedPosition;

    /**
     * 带状孪晶长度（新）
     */
    @Column(length = 11)
    private Integer twinBandedLength;

    /**
     * 隐裂距头位置（新）
     */
    @Column(length = 11)
    private Integer fissurePosition;

    /**
     * 隐裂长度（新）
     */
    @Column(length = 11)
    private Integer fissureLength;

    /**
     * 端面斜距头位置（新）
     */
    @Column(length = 11)
    private Integer obliquePosition;

    /**
     * 端面斜长度（新）
     */
    @Column(length = 11)
    private Integer obliqueLength;

    /**
     * 端面台阶距头位置（新）
     */
    @Column(length = 11)
    private Integer stepPosition;

    /**
     * 端面台阶长度（新）
     */
    @Column(length = 11)
    private Integer stepLength;

    /**
     * 棱线宽/分叉距头位置（新）
     */
    @Column(length = 11)
    private Integer crestPosition;

    /**
     * 棱线宽/分叉长度（新）
     */
    @Column(length = 11)
    private Integer crestLength;


    /**
     * A电阻率均值（新）
     */
    @Column(length = 19, name = "mean_resistivity_a")
    private BigDecimal meanResistivityA;

    /**
     * A电阻率1（新）
     */
    @Column(length = 19, name = "resistivity_a_one")
    private BigDecimal resistivityAOne;
    /**
     * A电阻率2（新）
     */
    @Column(length = 19, name = "resistivity_a_two")
    private BigDecimal resistivityATwo;
    /**
     * A电阻率3（新）
     */
    @Column(length = 19, name = "resistivity_a_three")
    private BigDecimal resistivityAThree;

    /**
     * B电阻率均值（新）
     */
    @Column(length = 19, name = "mean_resistivity_b")
    private BigDecimal meanResistivityB;

    /**
     * B电阻率1（新）
     */
    @Column(length = 19, name = "resistivity_b_one")
    private BigDecimal resistivityBOne;
    /**
     * B电阻率2（新）
     */
    @Column(length = 19, name = "resistivity_b_two")
    private BigDecimal resistivityBTwo;
    /**
     * B电阻率3（新）
     */
    @Column(length = 19, name = "resistivity_b_three")
    private BigDecimal resistivityBThree;

    /**
     * A少子寿命均值（新）
     */
    @Column(length = 19, name = "mean_carrier_lifetime_a")
    private BigDecimal meanCarrierLifetimeA;

    /**
     * A少子寿命1（新）
     */
    @Column(length = 19, name = "carrier_lifetime_a_one")
    private BigDecimal carrierLifetimeAOne;

    /**
     * A少子寿命2（新）
     */
    @Column(length = 19, name = "carrier_lifetime_a_two")
    private BigDecimal carrierLifetimeATwo;

    /**
     * A少子寿命3（新）
     */
    @Column(length = 19, name = "carrier_lifetime_a_three")
    private BigDecimal carrierLifetimeAThree;


    /**
     * B少子寿命均值（新）
     */
    @Column(length = 19, name = "mean_carrier_lifetime_b")
    private BigDecimal meanCarrierLifetimeB;

    /**
     * B少子寿命1（新）
     */
    @Column(length = 19, name = "carrier_lifetime_b_one")
    private BigDecimal carrierLifetimeBOne;

    /**
     * B少子寿命2（新）
     */
    @Column(length = 19, name = "carrier_lifetime_b_two")
    private BigDecimal carrierLifetimeBTwo;

    /**
     * B少子寿命3（新）
     */
    @Column(length = 19, name = "carrier_lifetime_b_three")
    private BigDecimal carrierLifetimeBThree;


    /**
     * T段电阻率（新）
     */
    @Column(length = 19)
    private BigDecimal tResistivity;

    /**
     * w段电阻率（新）
     */
    @Column(length = 19)
    private BigDecimal wResistivity;

    /**
     * 单晶拉制异常
     */
    @Column(length = 50)
    private String crystalAbnormal;

    /**
     * 完结标识
     */
    @Column(length = 50)
    private String closureIdentification;


    /**
     * 模块（拉晶车间）
     */
    @Column(length = 50)
    private String crystalWorkshop;

    /**
     * 片区（新）
     */
    @Column(length = 20)
    private String area;

    /**
     * 炉台类型【0:默认；0-量产;2-研发;】
     */
    @Column(length = 3)
    private Integer furnaceType;


    /**
     * 工厂
     */
    @Column(length = 100)
    private String factory;

    /**
     * 炉次号
     */
    @Column(length = 80)
    private String lotNo;

    /**
     * 工单类型(截断工单 )(新)
     */
    @Column(length = 80)
    private String workOrderType;

    /**
     * 工单编号(新)
     */
    @Column(length = 100)
    private String workOrderNo;


    /**
     * 物料编码
     */
    @Column(length = 150)
    private String materialCode;

    /**
     * 计划产品等级
     */
    @Column(length = 50)
    private String planGrade;

    /**
     * 计划掺杂剂
     */
    @Column(length = 20)
    private String planDopant;

    /**
     * 计划导电型号
     */
    @Column(length = 8)
    private String planConductiveType;


    /**
     * 计划规格
     */
    @Column(length = 50)
    private String planSpecs;

    /**
     * 产品简称(新)
     */
    @Column(length = 120)
    private String abbreviation;

    /**
     * 产品配方(新)
     */
    @Column(length = 120)
    private String formulation;


    /**
     * 评级标准版本
     */
    @Column(length = 50)
    private String gradeVersion;


    /**
     * 检验人员
     */
    @Column(length = 40)
    private String inspector;

    /**
     * 头段A电阻率均值
     */
    @Column(length = 19)
    private BigDecimal tRadialCentre;

    /**
     * 头段B电阻率均值
     */
    @Column(length = 19)
    private BigDecimal tRadialEdge1;

    /**
     * 尾段A电阻率均值
     */
    @Column(length = 19)
    private BigDecimal wRadialCentre;

    /**
     * 尾段B电阻率均值
     */
    @Column(length = 19)
    private BigDecimal wRadialEdge1;

    /**
     * 头段A少子寿命均值
     */
    @Column(length = 19)
    private BigDecimal tCentreLife;

    /**
     * 头段B少子寿命均值
     */
    @Column(length = 19)
    private BigDecimal tLifeEdge1;

    /**
     * 尾段A少子寿命均值
     */
    @Column(length = 19)
    private BigDecimal wCentreLife;

    /**
     * 尾段B少子寿命均值
     */
    @Column(length = 10)
    private BigDecimal wLifeEdge1;


    /**
     * 产品类型 毛棒划线时标记‘试加工’字段为‘是’时，所有晶段全部赋值‘产品类型’参数值为’试加工’
     */
    @Column(length = 20)
    private String productPattern;


    /**
     * 实验等级
     */
    @Column(length = 150)
    private String productLevelExperiment;

    /**
     * 圆棒最终评级/评级
     */
    @Column(length = 20)
    private String level;

    /**
     * 最终实验等级/实验等级
     */
    @Column(length = 20)
    private String experiLevel;


    /**
     * 崩边长度
     */
    @Column(length = 19)
    private BigDecimal edgeBark;

    /**
     * 斜【0：默认，否；1：是；】
     */
    @Column(length = 3)
    private Integer isBevel;

    /**
     * 斜长
     */
    @Column(length = 19)
    BigDecimal slopeLength;

    /**
     * 扭曲
     */
    @Column(length = 19)
    private BigDecimal distortion;


    /**
     * 复投次数
     */
    @Column(length = 10)
    private Integer againNum;

    /**
     * 颗数
     */
    @Column(length = 10)
    private Integer againPiece;

    /**
     * 径向电阻率T边缘2
     */
    @Column(length = 10)
    private BigDecimal tRadialEdge2;

    /**
     * 径向电阻率T边缘3
     */
    @Column(length = 10)
    private BigDecimal tRadialEdge3;

    /**
     * 径向电阻率T边缘4
     */
    @Column(length = 10)
    private BigDecimal tRadialEdge4;

    /**
     * 径向电阻率T边缘5
     */
    @Column(length = 10)
    private BigDecimal tRadialEdge5;

    /**
     * 径向电阻率T边缘6
     */
    @Column(length = 10)
    private BigDecimal tRadialEdge6;

    /**
     * 径向电阻率T边缘7
     */
    @Column(length = 10)
    private BigDecimal tRadialEdge7;

    /**
     * 径向电阻率T边缘8
     */
    @Column(length = 10)
    private BigDecimal tRadialEdge8;

    /**
     * 电阻率头均值
     */
    @Column(length = 19)
    private String resistivityTopAvg;

    /**
     * 径向电阻率W边缘2
     */
    @Column(length = 10)
    private BigDecimal wRadialEdge2;

    /**
     * 径向电阻率W边缘3
     */
    @Column(length = 10)
    private BigDecimal wRadialEdge3;

    /**
     * 径向电阻率W边缘4
     */
    @Column(length = 10)
    private BigDecimal wRadialEdge4;

    /**
     * 径向电阻率W边缘5
     */
    @Column(length = 10)
    private BigDecimal wRadialEdge5;

    /**
     * 径向电阻率W边缘6
     */
    @Column(length = 10)
    private BigDecimal wRadialEdge6;

    /**
     * 径向电阻率W边缘7
     */
    @Column(length = 10)
    private BigDecimal wRadialEdge7;

    /**
     * 径向电阻率W边缘8
     */
    @Column(length = 10)
    private BigDecimal wRadialEdge8;

    /**
     * T边缘寿命2
     */
    @Column(length = 10)
    private BigDecimal tLifeEdge2;

    /**
     * T边缘寿命3
     */
    @Column(length = 10)
    private BigDecimal tLifeEdge3;

    /**
     * T边缘寿命4
     */
    @Column(length = 10)
    private BigDecimal tLifeEdge4;

    /**
     * W边缘寿命2
     */
    @Column(length = 10)
    private BigDecimal wLifeEdge2;

    /**
     * W边缘寿命3
     */
    @Column(length = 10)
    private BigDecimal wLifeEdge3;

    /**
     * W边缘寿命4
     */
    @Column(length = 10)
    private BigDecimal wLifeEdge4;

    /**
     * 少子寿命
     */
    private BigDecimal life;

    /**
     * T氧含量中心
     */
    @Column(length = 19)
    private BigDecimal oxygenContentHead;

    /**
     * W氧含量中心
     */
    @Column(length = 19)
    private BigDecimal oxygenContentTail;

    /**
     * 【完结尾】T氧含量中心
     */
    @Column(length = 19)
    private BigDecimal endOxygenContentHead;

    /**
     * 【完结尾】W氧含量中心
     */
    @Column(length = 19)
    private BigDecimal endOxygenContentTail;

    /**
     * T碳含量中心
     */
    @Column(length = 19)
    private BigDecimal carbonContentHead;

    /**
     * W碳含量中心
     */
    @Column(length = 19)
    private BigDecimal carbonContentTail;

    /**
     * 【完结尾】T碳含量中心
     */
    @Column(length = 19)
    private BigDecimal endCarbonContentHead;

    /**
     * 【完结尾】W碳含量中心
     */
    @Column(length = 19)
    private BigDecimal endCarbonContentTail;


    /**
     * 单晶头方向
     */
    @Column(length = 10)
    private String crystalDirection;

    /**
     * 是否转规格【0：默认，否；1：是；】
     */
    @Column(length = 3)
    private Integer isChangeSpecs;

    /**
     * 拉制状态【Y：表示不断苞;N：表示断苞;W：表示未断苞见埚底单晶;V：表示断苞见埚底单晶】
     */
    @Column(length = 4)
    private String pullType;


    /**
     * 炉台号
     */
    @Column(length = 50)
    private String deviceNo;


    /**
     * 订单类型（1:量产；2:试验；3:送样;4:返工）
     */
    @Column(length = 11)
    private String orderType;

    /**
     * 订单号
     */
    @Column(length = 150)
    private String orderNo;


    /**
     * 圆棒备注
     */
    @Column(length = 150)
    private String remark;

    /**
     * 更新人
     */
    @Column(length = 50)
    private String updateUser;

    /**
     * 更新时间
     */
    @Column(length = 19)
    @Temporal(TemporalType.TIMESTAMP)
    private Date updateTime;

    /**
     * 裂纹
     */
    @Column(length = 19)
    private BigDecimal crack;


    // 评级数据
    @Column(length = 100)
    private String gradeData;
    /**
     * 评级类型
     */
    @Column(length = 50)
    private String gradeType;

    /**
     * 是否反切【0：默认，否；1：是；】
     */
    @Column(nullable = false, length = 10)
    private Integer isCutBack;

    /**
     * 崩晶线
     */
    @Column(length = 19)
    private BigDecimal collapseLine;
    /**
     * 产品类别
     */
    @Column(length = 20)
    private String productType;

    /**
     * W寿命min
     */
    @Column(length = 19)
    private BigDecimal wLifeMin;

    //    /**
//     * 扭过大(0否 1是)
//     */
//    @Column(length = 3)
//    private Integer distortionBig;
//    /**
//     * 系列
//     */
//    @Column(length = 50)
//    private String series;
//
//    /**
//     * 库别【0-正常；1-直径不良单晶；2-退料；】
//     */
//    @Column(length = 20)
//    private String kube;
//
//    /**
//     * 入库长度
//     */
//    @Column(length = 19)
//    private BigDecimal wareHouseLength;
//


    /**
     * 方棒边长
     */
    @Column(length = 50)
    private String sideLengthRange;

    /**
     * 氧碳依赖标识：0：中间段；1：头段依赖；2：尾段依赖
     */
    @Column(length = 50)
    private Integer oxygenCarbonMark;

    /**
     * 首段A面氧含量
     */
    @Column(length = 19)
    private BigDecimal oxygenContentFirsta;

    /**
     * 首段A面碳含量
     */
    @Column(length = 19)
    private BigDecimal carbonContentFirsta;

    /**
     * 尾段B面氧含量
     */
    @Column(length = 19)
    private BigDecimal oxygenContentEndb;
    /**
     * 尾段B面碳含量
     */
    @Column(length = 19)
    private BigDecimal carbonContentEndb;


    /**
     * 批次号
     */
    @Column(length = 50)
    private String batchNo;

    /**
     * 工单定向标识
     */
    @Column
    private String zorientationind;

    /**
     * 性能返切次数
     */
    @Column(length = 2)
    private Integer behaviorFrequency;

    /**
     * 外观返切次数
     */
    @Column(length = 2)
    private Integer facadeFrequency;

    /**
     * 端面斜A
     */
    @Column(length = 20, name = "oblique_a")
    private Integer obliqueA;
    /**
     * 端面斜B
     */
    @Column(length = 20, name = "oblique_b")
    private Integer obliqueB;

//
//        if (headFlag){
//        //T氧含量中心
//        ybInspectData.setOxygenContentHead(pieceCheck.getoContents());
//        //T碳含量中心
//        ybInspectData.setCarbonContentHead(pieceCheck.getcContents());
//    }else {
//        //W氧含量中心
//        ybInspectData.setOxygenContentTail(pieceCheck.getoContents());
//        //W碳含量中心
//        ybInspectData.setCarbonContentTail(pieceCheck.getcContents());
//    }


    //
//
    // 头温度
    @Column(length = 19)
    private String temperature;
//
//    // 尾温度
//    @Column(length = 19)
//    private String temperatureEnd;
//
//    // 直径差
//    @Column(length = 8)
//    private String demcMax;
//
//    // 直径差大
//    @Column(length = 8)
//    private String diameterMinusBig;
//
//
//    /**
//     * 平均直径
//     */
//    @Column(length = 19)
//    private BigDecimal avgDiameter;
//
//    /**
//     * 整颗平均直径
//     */
//    @Column(length = 19)
//    private BigDecimal allAvgDiameter;
//
//    /**
//     * 直径判定
//     */
//    @Column(length = 80)
//    private String diamDecide;
//
//    /**
//     * 细/细总长
//     */
//    @Column(length = 19)
//    private BigDecimal fineDiameter;
//
//    /**
//     * 细减长
//     */
//    @Column(length = 19)
//    private BigDecimal fineCutLength;
//
//    /**
//     * 晶线分叉【0：默认，否；1：是；】
//     */
//    @Column(length = 3)
//    private Integer isFork;
//
//
//    /**
//     * 孪晶【0：默认，否；1：是；】
//     */
//    @Column(length = 3)
//    private Integer isTwinCrystal;
//
//    /**
//     * 孪晶长度
//     */
//    @Column(length = 19)
//    private BigDecimal twinLength;
//
//    /**
//     * 头斜
//     */
//    @Column(length = 19)
//    private BigDecimal headSlope;
//
//    /**
//     * 尾斜
//     */
//    @Column(length = 19)
//    private BigDecimal tailSlope;
//
//    /**
//     * 电阻率范围min
//     */
//    @Column(length = 10)
//    private BigDecimal resistivityMin;
//
//    /**
//     * 电阻率范围max
//     */
//    @Column(length = 10)
//    private BigDecimal resistivityMax;
//
//    /**
//     * T寿命min
//     */
//    @Column(length = 19)
//    private BigDecimal tLifeMin;
//

//
//
//    /**
//     * 电阻率头尾中心边缘4点均值
//     */
//    @Column(length = 19)
//    private BigDecimal endCentreEdgeAvg4;
//
//    /**
//     * 电阻率头中心边缘平均
//     */
//    @Column(length = 19)
//    private BigDecimal topCentreEdgeAvg;
//
//    /**
//     * 电阻率尾中心边缘平均
//     */
//    @Column(length = 19)
//    private BigDecimal endCentreEdgeAvg;
//
//
//    /**
//     * 电阻率尾均值
//     */
//    @Column(length = 19)
//    private BigDecimal resistivityEndAvg;
//
//    /**
//     * 出货电阻率
//     */
//    @Column(length = 19)
//    private BigDecimal sellResistivity;
//
//    /**
//     * 电阻率结论
//     */
//    @Column(length = 50)
//    private String resistivityConclusion;
//
//    /**
//     * 是否见锅底 1是0否
//     */
//    @Column(length = 3)
//    private Integer isSeeBottom;
//
//    /**
//     * 电阻率反翘
//     */
//    @Column(length = 32)
//    private String resistivityWarping;
//
//    /**
//     * T氧含量边缘
//     */
//    @Column(length = 19)
//    private BigDecimal oxygenContentBorder;
//
//    /**
//     * T碳含量边缘
//     */
//    @Column(length = 19)
//    private BigDecimal carbonContentBorder;
//
//    /**
//     * W氧含量边缘
//     */
//    @Column(length = 19)
//    private BigDecimal oxygenContentBorderTail;
//
//
//    /**
//     * W碳含量边缘
//     */
//    @Column(length = 19)
//    private BigDecimal carbonContentBorderTail;
//
//    /**
//     * T缺陷
//     */
//    @Column(length = 150)
//    private String headDefectGrade;
//
//    /**
//     * W缺陷
//     */
//    @Column(length = 150)
//    private String tailDefect;
//
//
//    // 头位错
//    @Column(length = 3)
//    Integer isMalpositionHead;
//
//    // 尾位错
//    @Column(length = 3)
//    Integer isMalpositionTail;
//
//    /**
//     * T样片中心退火电阻率
//     */
//    @Column(length = 10)
//    private BigDecimal tCenterAnneal;
//
//    /**
//     * W样片中心退火电阻率
//     */
//    @Column(length = 10)
//    private BigDecimal wcenterAnneal;
//
//    /**
//     * T样片边缘退火电阻率
//     */
//    @Column(length = 10)
//    private BigDecimal tBorderAnneal;
//
//    /**
//     * W样片边缘退火电阻率
//     */
//    @Column(length = 10)
//    private BigDecimal wBorderAnneal;
//
//    /**
//     * 异常汇总
//     */
//    @Column(length = 150)
//    private String anomaly;
//
//    /**
//     * 检验日期
//     */
//    @Column(length = 50)
//    private String hasGone;
//
//    /**
//     * 计划订单号
//     */
//    @Column(length = 150)
//    private String planOrderNo;
//
//    /**
//     * 计划物料编码
//     */
//    @Column(length = 50)
//    private String planMaterialCode;
//
//
//    /**
//     * 投料量
//     */
//    @Column(length = 19)
//    private Integer materialCount;
//
//    /**
//     * T寿命/电阻率中心
//     */
//    @Column(length = 19)
//    private BigDecimal sintonCenter;
//
//    /**
//     * W寿命/电阻率中心
//     */
//    @Column(length = 19)
//    private BigDecimal sintonTail;
//
//    /**
//     * 01W寿命
//     */
//    @Column(length = 10)
//    private Integer lifeSpanw1;
//
//    /**
//     * 01W电阻率
//     */
//    @Column(length = 19)
//    private BigDecimal resistivityw1;
//
//    /**
//     * 02T寿命
//     */
//    @Column(length = 10)
//    private Integer lifeSpant2;
//
//    /**
//     * 02T电阻率
//     */
//    @Column(length = 8)
//    private BigDecimal resistivityt2;
//
//
//    /**
//     * 画线实验标记
//     */
//    @Column(length = 10)
//    private String drawLine;
//
//
//    /**
//     * 头部评级
//     */
//    @Column(length = 60)
//    private String headLevel;
//
//    /**
//     * 尾部评级
//     */
//    @Column(length = 60)
//    private String tailLevel;
//
//    /**
//     * 评级标准名称
//     */
//    @Column(length = 50)
//    private String gradeName;
//
//    /**
//     * 实验等级标准名称
//     */
//    @Column(length = 50)
//    private String planGradeName;
//
//
//    // 计划拼数
//    @Column(length = 20)
//    private String planStickCount;
//
//    // 计划长度范围
//    @Column(length = 20)
//    private String planLenRange;
//
//    /**
//     * 实验等级标准版本
//     */
//    @Column(length = 50)
//    private String planGradeVersion;
//
//
////	/**
////	 * 温度补偿值
////	 */
////    @Column(length=10)
////    private BigDecimal tempValue;
//
//    /**
//     * 温度补偿后径向电阻率T中心点
//     */
//    @Column(length = 10)
//    private BigDecimal tTempRadialCentre;
//
//    /**
//     * 温度补偿后径向电阻率T边缘1
//     */
//    @Column(length = 10)
//    private BigDecimal tTempRadialEdge1;
//
//    /**
//     * 温度补偿后径向电阻率T边缘2
//     */
//    @Column(length = 10)
//    private BigDecimal tTempRadialEdge2;
//
//    /**
//     * 温度补偿后径向电阻率T边缘3
//     */
//    @Column(length = 10)
//    private BigDecimal tTempRadialEdge3;
//
//    /**
//     * 温度补偿后径向电阻率T边缘4
//     */
//    @Column(length = 10)
//    private BigDecimal tTempRadialEdge4;
//
//    /**
//     * 温度补偿后径向电阻率T边缘5
//     */
//    @Column(length = 10)
//    private BigDecimal tTempRadialEdge5;
//
//    /**
//     * 温度补偿后径向电阻率T边缘6
//     */
//    @Column(length = 10)
//    private BigDecimal tTempRadialEdge6;
//
//    /**
//     * 温度补偿后径向电阻率T边缘7
//     */
//    @Column(length = 10)
//    private BigDecimal tTempRadialEdge7;
//
//    /**
//     * 温度补偿后径向电阻率T边缘8
//     */
//    @Column(length = 10)
//    private BigDecimal tTempRadialEdge8;
//
//    /**
//     * 温度补偿后径向电阻率W中心点
//     */
//    @Column(length = 10)
//    private BigDecimal wTempRadialCentre;
//
//    /**
//     * 温度补偿后径向电阻率W边缘1
//     */
//    @Column(length = 10)
//    private BigDecimal wTempRadialEdge1;
//
//    /**
//     * 温度补偿后径向电阻率W边缘2
//     */
//    @Column(length = 10)
//    private BigDecimal wTempRadialEdge2;
//
//    /**
//     * 温度补偿后径向电阻率W边缘3
//     */
//    @Column(length = 10)
//    private BigDecimal wTempRadialEdge3;
//
//    /**
//     * 温度补偿后径向电阻率W边缘4
//     */
//    @Column(length = 10)
//    private BigDecimal wTempRadialEdge4;
//
//    /**
//     * 温度补偿后径向电阻率W边缘5
//     */
//    @Column(length = 10)
//    private BigDecimal wTempRadialEdge5;
//
//    /**
//     * 温度补偿后径向电阻率W边缘6
//     */
//    @Column(length = 10)
//    private BigDecimal wTempRadialEdge6;
//
//    /**
//     * 温度补偿后径向电阻率W边缘7
//     */
//    @Column(length = 10)
//    private BigDecimal wTempRadialEdge7;
//
//    /**
//     * 温度补偿后径向电阻率W边缘8
//     */
//    @Column(length = 10)
//    private BigDecimal wTempRadialEdge8;
//
//
//    // 计划订单类型
//    @Column(length = 8)
//    private String planOrderType;
//
//    /**
//     * 划线实验标记
//     */
//    @Column(length = 200)
//    private String lineMarking;
//
//    /**
//     * 是否带彩 默认 否
//     */
//    @Column(length = 3)
//    private Boolean colorful;
//
//    /**
//     * 补偿01W电阻率
//     */
//    @Column(length = 10)
//    private BigDecimal compensateResistivityw1;
//
//    /**
//     * 补偿02T电阻率
//     */
//    @Column(length = 10)
//    private BigDecimal compensateResistivityt2;

    public YbInspectData() {
        this.distortion = BigDecimal.ZERO;
        this.slopeLength = BigDecimal.ZERO;
        this.edgeBark = BigDecimal.ZERO;
        this.crack = BigDecimal.ZERO;
    }

    public YbInspectData(Long id) {
        this.id = id;
    }
}
