package com.sw.mes.kanban.controller;

import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;
import com.sw.mes.kanban.entity.KanbanDevice;
import com.sw.mes.kanban.dto.KanbanDeviceDto;
import com.sw.mes.kanban.service.KanbanDeviceService;


/**
* @Author
* @Description：
* @Date: Create in
* @Modified By:
*/
@RestController
@RequestMapping("/kanbanDevice")
@Slf4j
@Api(description = "KanbanDevice接口")
public class KanbanDeviceController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private KanbanDeviceService kanbanDeviceService;

	@ApiOperation(value = "分页查询KanbanDevice列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<KanbanDeviceDto>> listByPage(ServletRequest request,@ApiIgnore RequestPage requestPage) {
		Specification<KanbanDevice> specification = DynamicSpecifications.bySearchFilter(request, KanbanDevice.class);
    	Page<KanbanDevice> page = kanbanDeviceService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<KanbanDeviceDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有KanbanDevice")
	@GetMapping("/listAll")
	public Result<List<KanbanDeviceDto>> listAll(ServletRequest request) {
		Specification<KanbanDevice> specification = DynamicSpecifications.bySearchFilter(request, KanbanDevice.class);
		List<KanbanDevice> kanbanDevices = kanbanDeviceService.findByExampleSort(specification, null);
		List<KanbanDeviceDto> kanbanDeviceDtos = kanbanDevices.stream()
					.map(kanbanDevice -> toLazyDto(kanbanDevice))
					.collect(Collectors.toList());
		return ResultGenerator.success(kanbanDeviceDtos);
	}

	@ApiOperation(value = "KanbanDevice详细信息")
	@GetMapping("/{id}")
	public Result<KanbanDeviceDto> get(@PathVariable @ApiParam(value="KanbanDeviceid") Long id) {
		KanbanDevice kanbanDevice = kanbanDeviceService.get(id);
		if (null == kanbanDevice)
			return ResultGenerator.fail("该KanbanDevice不存在");
		return ResultGenerator.success(toEagerDto(kanbanDevice));
	}

	@ApiOperation(value = "创建KanbanDevice")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "KanbanDevice对象") KanbanDeviceDto kanbanDeviceDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		KanbanDevice kanbanDevice = fromDto(kanbanDeviceDto);
		kanbanDeviceService.saveOrUpdate(kanbanDevice);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "更新KanbanDevice")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "KanbanDevice") KanbanDeviceDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors())
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        if (dto.getId() == null)
            return ResultGenerator.fail("id不能为空");
            KanbanDevice entity = kanbanDeviceService.get(dto.getId());
        if (entity == null)
            return ResultGenerator.fail("原信息不存在");
        try {
            KanbanDevice entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			kanbanDeviceService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除KanbanDevice")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
		for (Integer id : ids) {
				kanbanDeviceService.delete(id);
		}
		return ResultGenerator.success();
	}

	public KanbanDeviceDto toLazyDto(KanbanDevice entity) {
		return modelMapper.map(entity, KanbanDeviceDto.class);
	}

	public KanbanDeviceDto toEagerDto(KanbanDevice entity) {
		KanbanDeviceDto dto = modelMapper.map(entity, KanbanDeviceDto.class);
		return dto;
	}

	public KanbanDevice fromDto(KanbanDeviceDto dto) {
		return modelMapper.map(dto, KanbanDevice.class);
	}


}
