package com.sw.mes.outside.controller;

import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.*;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;
import com.sw.mes.outside.entity.AlarmConfig;
import com.sw.mes.outside.dto.AlarmConfigDto;
import com.sw.mes.outside.service.AlarmConfigService;


/**
 * @Author
 * @Description：
 * @Date: Create in
 * @Modified By:
 */
@RestController
@RequestMapping("/AlarmConfig")
@Slf4j
@Api(description = "报警配置联接口")
public class AlarmConfigController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private AlarmConfigService alarmConfigService;

	@ApiOperation(value = "分页查询报警配置联列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<AlarmConfigDto>> listByPage(ServletRequest request,@ApiIgnore RequestPage requestPage) {
		Specification<AlarmConfig> specification = DynamicSpecifications.bySearchFilter(request, AlarmConfig.class);
    	Page<AlarmConfig> page = alarmConfigService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<AlarmConfigDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有报警配置联")
	@GetMapping("/listAll")
	public Result<List<AlarmConfigDto>> listAll(ServletRequest request) {
		Specification<AlarmConfig> specification = DynamicSpecifications.bySearchFilter(request, AlarmConfig.class);
		List<AlarmConfig> alarmConfigs = alarmConfigService.findByExampleSort(specification, null);
		List<AlarmConfigDto> alarmConfigDtos = alarmConfigs.stream()
					.map(alarmConfig -> toLazyDto(alarmConfig))
					.collect(Collectors.toList());
		return ResultGenerator.success(alarmConfigDtos);
	}

	@ApiOperation(value = "报警配置联详细信息")
	@GetMapping("/{id}")
	public Result<AlarmConfigDto> get(@PathVariable @ApiParam(value="报警配置联id") Long id) {
		AlarmConfig alarmConfig = alarmConfigService.get(id);
		if (null == alarmConfig){
			return ResultGenerator.fail("该报警配置联不存在");
		}
		return ResultGenerator.success(toEagerDto(alarmConfig));
	}

	@ApiOperation(value = "创建报警配置联")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "报警配置联对象") AlarmConfigDto alarmConfigDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		AlarmConfig alarmConfig = fromDto(alarmConfigDto);
		alarmConfigService.saveOrUpdate(alarmConfig);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "更新报警配置联")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "报警配置联") AlarmConfigDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors()){
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
        if (dto.getId() == null){
            return ResultGenerator.fail("id不能为空");
		}
		AlarmConfig entity = alarmConfigService.get(dto.getId());
        if (entity == null){
            return ResultGenerator.fail("原信息不存在");
		}
        try {
            AlarmConfig entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			alarmConfigService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除报警配置联")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
		for (Integer id : ids) {
				alarmConfigService.delete(id);
		}
		return ResultGenerator.success();
	}

	public AlarmConfigDto toLazyDto(AlarmConfig entity) {
		return modelMapper.map(entity, AlarmConfigDto.class);
	}

	public AlarmConfigDto toEagerDto(AlarmConfig entity) {
		AlarmConfigDto dto = modelMapper.map(entity, AlarmConfigDto.class);
		return dto;
	}

	public AlarmConfig fromDto(AlarmConfigDto dto) {
		return modelMapper.map(dto, AlarmConfig.class);
	}


}
