package com.sw.mes.sap.controller;


import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import com.sw.mes.sap.dto.ReportErpInputRecordDetailDto;
import com.sw.mes.sap.entity.ReportErpInputRecordDetail;
import com.sw.mes.sap.service.ReportErpInputRecordDetailService;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;


/**
* @Author
* @Description：
* @Date: Create in
* @Modified By:
*/
@RestController
@RequestMapping("/ReportErpInputRecordDetail")
@Slf4j
@Api(description = "拉取投料上报erp记录子表接口")
public class ReportErpInputRecordDetailController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private ReportErpInputRecordDetailService reportErpInputRecordDetailService;

	@ApiOperation(value = "分页查询拉取投料上报erp记录子表列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<ReportErpInputRecordDetailDto>> listByPage(ServletRequest request, @ApiIgnore RequestPage requestPage) {
		Specification<ReportErpInputRecordDetail> specification = DynamicSpecifications.bySearchFilter(request, ReportErpInputRecordDetail.class);
    	Page<ReportErpInputRecordDetail> page = reportErpInputRecordDetailService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<ReportErpInputRecordDetailDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有拉取投料上报erp记录子表")
	@GetMapping("/listAll")
	public Result<List<ReportErpInputRecordDetailDto>> listAll(ServletRequest request) {
		Specification<ReportErpInputRecordDetail> specification = DynamicSpecifications.bySearchFilter(request, ReportErpInputRecordDetail.class);
		List<ReportErpInputRecordDetail> reportErpInputRecordDetails = reportErpInputRecordDetailService.findByExampleSort(specification, null);
		List<ReportErpInputRecordDetailDto> reportErpInputRecordDetailDtos = reportErpInputRecordDetails.stream()
					.map(reportErpInputRecordDetail -> toLazyDto(reportErpInputRecordDetail))
					.collect(Collectors.toList());
		return ResultGenerator.success(reportErpInputRecordDetailDtos);
	}

	@ApiOperation(value = "拉取投料上报erp记录子表详细信息")
	@GetMapping("/{id}")
	public Result<ReportErpInputRecordDetailDto> get(@PathVariable @ApiParam(value="拉取投料上报erp记录子表id") Long id) {
		ReportErpInputRecordDetail reportErpInputRecordDetail = reportErpInputRecordDetailService.get(id);
		if (null == reportErpInputRecordDetail)
			return ResultGenerator.fail("该拉取投料上报erp记录子表不存在");
		return ResultGenerator.success(toEagerDto(reportErpInputRecordDetail));
	}

	@ApiOperation(value = "创建拉取投料上报erp记录子表")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "拉取投料上报erp记录子表对象") ReportErpInputRecordDetailDto reportErpInputRecordDetailDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		ReportErpInputRecordDetail reportErpInputRecordDetail = fromDto(reportErpInputRecordDetailDto);
		reportErpInputRecordDetailService.saveOrUpdate(reportErpInputRecordDetail);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "更新拉取投料上报erp记录子表")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "拉取投料上报erp记录子表") ReportErpInputRecordDetailDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors())
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        if (dto.getId() == null)
            return ResultGenerator.fail("id不能为空");
            ReportErpInputRecordDetail entity = reportErpInputRecordDetailService.get(dto.getId());
        if (entity == null)
            return ResultGenerator.fail("原信息不存在");
        try {
            ReportErpInputRecordDetail entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			reportErpInputRecordDetailService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除拉取投料上报erp记录子表")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
		for (Integer id : ids) {
				reportErpInputRecordDetailService.delete(id);
		}
		return ResultGenerator.success();
	}

	public ReportErpInputRecordDetailDto toLazyDto(ReportErpInputRecordDetail entity) {
		return modelMapper.map(entity, ReportErpInputRecordDetailDto.class);
	}

	public ReportErpInputRecordDetailDto toEagerDto(ReportErpInputRecordDetail entity) {
		ReportErpInputRecordDetailDto dto = modelMapper.map(entity, ReportErpInputRecordDetailDto.class);
		return dto;
	}

	public ReportErpInputRecordDetail fromDto(ReportErpInputRecordDetailDto dto) {
		return modelMapper.map(dto, ReportErpInputRecordDetail.class);
	}


}
