package com.sw.mes.sap.controller;


import com.alibaba.excel.util.CollectionUtils;
import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import com.sw.mes.sap.dto.ReportErpSquareOutputRecordDetailDto;
import com.sw.mes.sap.entity.ReportErpSquareOutputRecordDetail;
import com.sw.mes.sap.service.ReportErpSquareOutputRecordDetailService;
import com.sw.mes.yb.production.entity.LongCrystal;
import com.sw.mes.yb.production.service.LongCrystalService;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;


/**
* @Author
* @Description：
* @Date: Create in
* @Modified By:
*/
@RestController
@RequestMapping("/ReportErpSquareOutputRecordDetail")
@Slf4j
@Api(description = "拉取长晶棒产出信息【已切断】上报erp记录子表接口")
public class ReportErpSquareOutputRecordDetailController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private ReportErpSquareOutputRecordDetailService reportErpSquareOutputRecordDetailService;

	@Autowired
	private LongCrystalService longCrystalService;

	@ApiOperation(value = "分页查询拉取长晶棒产出信息【已切断】上报erp记录子表列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<ReportErpSquareOutputRecordDetailDto>> listByPage(ServletRequest request, @ApiIgnore RequestPage requestPage) {
		Specification<ReportErpSquareOutputRecordDetail> specification = DynamicSpecifications.bySearchFilter(request, ReportErpSquareOutputRecordDetail.class);
    	Page<ReportErpSquareOutputRecordDetail> page = reportErpSquareOutputRecordDetailService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<ReportErpSquareOutputRecordDetailDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有拉取长晶棒产出信息【已切断】上报erp记录子表")
	@GetMapping("/listAll")
	public Result<List<ReportErpSquareOutputRecordDetailDto>> listAll(ServletRequest request) {
		Specification<ReportErpSquareOutputRecordDetail> specification = DynamicSpecifications.bySearchFilter(request, ReportErpSquareOutputRecordDetail.class);
		List<ReportErpSquareOutputRecordDetail> reportErpSquareOutputRecordDetails = reportErpSquareOutputRecordDetailService.findByExampleSort(specification, null);
		List<ReportErpSquareOutputRecordDetailDto> reportErpSquareOutputRecordDetailDtos = reportErpSquareOutputRecordDetails.stream()
					.map(reportErpSquareOutputRecordDetail -> toLazyDto(reportErpSquareOutputRecordDetail))
					.collect(Collectors.toList());
		return ResultGenerator.success(reportErpSquareOutputRecordDetailDtos);
	}

	@ApiOperation(value = "拉取长晶棒产出信息【已切断】上报erp记录子表详细信息")
	@GetMapping("/{id}")
	public Result<List<ReportErpSquareOutputRecordDetailDto>> get(@PathVariable @ApiParam(value="拉取长晶棒产出信息【已切断】上报erp记录子表id") Long id) {
		List<ReportErpSquareOutputRecordDetail> list = reportErpSquareOutputRecordDetailService.findByRid(id);
		if(CollectionUtils.isEmpty(list)){
			return ResultGenerator.fail("该拉取长晶棒产出信息上报erp记录子表不存在");
		}
		List<ReportErpSquareOutputRecordDetailDto> reportErpSquareOutputRecordDetailDtos = list.stream()
				.map(reportErpOutputRecordDetail -> toLazyDto(reportErpOutputRecordDetail))
				.collect(Collectors.toList());
		return ResultGenerator.success(reportErpSquareOutputRecordDetailDtos);
	}

	@ApiOperation(value = "创建拉取长晶棒产出信息【已切断】上报erp记录子表")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "拉取长晶棒产出信息【已切断】上报erp记录子表对象") ReportErpSquareOutputRecordDetailDto reportErpSquareOutputRecordDetailDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		ReportErpSquareOutputRecordDetail reportErpSquareOutputRecordDetail = fromDto(reportErpSquareOutputRecordDetailDto);
		reportErpSquareOutputRecordDetailService.saveOrUpdate(reportErpSquareOutputRecordDetail);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "更新拉取长晶棒产出信息【已切断】上报erp记录子表")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "拉取长晶棒产出信息【已切断】上报erp记录子表") ReportErpSquareOutputRecordDetailDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors())
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        if (dto.getId() == null)
            return ResultGenerator.fail("id不能为空");
            ReportErpSquareOutputRecordDetail entity = reportErpSquareOutputRecordDetailService.get(dto.getId());
        if (entity == null)
            return ResultGenerator.fail("原信息不存在");
        try {
            ReportErpSquareOutputRecordDetail entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			reportErpSquareOutputRecordDetailService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除拉取长晶棒产出信息【已切断】上报erp记录子表")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
		for (Integer id : ids) {
			reportErpSquareOutputRecordDetailService.delete(id);
		}
		return ResultGenerator.success();
	}

	@ApiOperation(value = "更新长晶棒产出信息【已切断】拉取状态")
	@PostMapping("/squarePulled")
	public Result updateSquarePulled(@RequestParam("squarePulled") String crystalNo) {
		LongCrystal byCrystalNo = longCrystalService.findByCrystalNo(crystalNo);
		if(null != byCrystalNo){
			byCrystalNo.setSquarePulled(0);
			longCrystalService.saveOrUpdate(byCrystalNo);
		}
		return ResultGenerator.success();
	}

	public ReportErpSquareOutputRecordDetailDto toLazyDto(ReportErpSquareOutputRecordDetail entity) {
		return modelMapper.map(entity, ReportErpSquareOutputRecordDetailDto.class);
	}

	public ReportErpSquareOutputRecordDetailDto toEagerDto(ReportErpSquareOutputRecordDetail entity) {
		ReportErpSquareOutputRecordDetailDto dto = modelMapper.map(entity, ReportErpSquareOutputRecordDetailDto.class);
		return dto;
	}

	public ReportErpSquareOutputRecordDetail fromDto(ReportErpSquareOutputRecordDetailDto dto) {
		return modelMapper.map(dto, ReportErpSquareOutputRecordDetail.class);
	}


}
