package com.sw.mes.yb.alloy.controller;

import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;
import com.sw.mes.yb.alloy.entity.FixLogicHisLog;
import com.sw.mes.yb.alloy.dto.FixLogicHisLogDto;
import com.sw.mes.yb.alloy.service.FixLogicHisLogService;


/**
* @Author
* @Description：
* @Date: Create in
* @Modified By:
*/
@RestController
@RequestMapping("/fixLogicHisLog")
@Slf4j
@Api(description = "修正逻辑历史记录表接口")
public class FixLogicHisLogController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private FixLogicHisLogService fixLogicHisLogService;

	@ApiOperation(value = "分页查询修正逻辑历史记录表列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<FixLogicHisLogDto>> listByPage(ServletRequest request,@ApiIgnore RequestPage requestPage) {
		Specification<FixLogicHisLog> specification = DynamicSpecifications.bySearchFilter(request, FixLogicHisLog.class);
    	Page<FixLogicHisLog> page = fixLogicHisLogService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<FixLogicHisLogDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有修正逻辑历史记录表")
	@GetMapping("/listAll")
	public Result<List<FixLogicHisLogDto>> listAll(ServletRequest request) {
		Specification<FixLogicHisLog> specification = DynamicSpecifications.bySearchFilter(request, FixLogicHisLog.class);
		List<FixLogicHisLog> fixLogicHisLogs = fixLogicHisLogService.findByExampleSort(specification, null);
		List<FixLogicHisLogDto> fixLogicHisLogDtos = fixLogicHisLogs.stream()
					.map(fixLogicHisLog -> toLazyDto(fixLogicHisLog))
					.collect(Collectors.toList());
		return ResultGenerator.success(fixLogicHisLogDtos);
	}

	@ApiOperation(value = "修正逻辑历史记录表详细信息")
	@GetMapping("/{id}")
	public Result<FixLogicHisLogDto> get(@PathVariable @ApiParam(value="修正逻辑历史记录表id") Integer id) {
		FixLogicHisLog fixLogicHisLog = fixLogicHisLogService.get(id);
		if (null == fixLogicHisLog)
			return ResultGenerator.fail("该修正逻辑历史记录表不存在");
		return ResultGenerator.success(toEagerDto(fixLogicHisLog));
	}

	@ApiOperation(value = "创建修正逻辑历史记录表")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "修正逻辑历史记录表对象") FixLogicHisLogDto fixLogicHisLogDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		FixLogicHisLog fixLogicHisLog = fromDto(fixLogicHisLogDto);
		fixLogicHisLogService.saveOrUpdate(fixLogicHisLog);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "更新修正逻辑历史记录表")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "修正逻辑历史记录表") FixLogicHisLogDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors())
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        if (dto.getId() == null)
            return ResultGenerator.fail("id不能为空");
            FixLogicHisLog entity = fixLogicHisLogService.get(dto.getId());
        if (entity == null)
            return ResultGenerator.fail("原信息不存在");
        try {
            FixLogicHisLog entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			fixLogicHisLogService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除修正逻辑历史记录表")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
		for (Integer id : ids) {
				fixLogicHisLogService.delete(id);
		}
		return ResultGenerator.success();
	}

	public FixLogicHisLogDto toLazyDto(FixLogicHisLog entity) {
		return modelMapper.map(entity, FixLogicHisLogDto.class);
	}

	public FixLogicHisLogDto toEagerDto(FixLogicHisLog entity) {
		FixLogicHisLogDto dto = modelMapper.map(entity, FixLogicHisLogDto.class);
		return dto;
	}

	public FixLogicHisLog fromDto(FixLogicHisLogDto dto) {
		return modelMapper.map(dto, FixLogicHisLog.class);
	}


}
