package com.sw.mes.yb.check.controller;

import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.*;
import com.sw.mes.core.utils.RequestPage;
import com.sw.mes.yb.check.dto.OxygenReversalLogDto;
import com.sw.mes.yb.check.entity.OxygenReversalLog;
import com.sw.mes.yb.check.service.OxygenReversalLogService;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;


/**
 * @Author
 * @Description：
 * @Date: Create in
 * @Modified By:
 */
@RestController
@RequestMapping("/OxygenReversalLog")
@Slf4j
@Api(description = "氧反翘记录接口")
public class OxygenReversalLogController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private OxygenReversalLogService oxygenReversalLogService;

	@ApiOperation(value = "分页查询氧反翘记录列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<OxygenReversalLogDto>> listByPage(ServletRequest request, @ApiIgnore RequestPage requestPage) {
		Specification<OxygenReversalLog> specification = DynamicSpecifications.bySearchFilter(request, OxygenReversalLog.class);
    	Page<OxygenReversalLog> page = oxygenReversalLogService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<OxygenReversalLogDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有氧反翘记录")
	@GetMapping("/listAll")
	public Result<List<OxygenReversalLogDto>> listAll(ServletRequest request) {
		Specification<OxygenReversalLog> specification = DynamicSpecifications.bySearchFilter(request, OxygenReversalLog.class);
		List<OxygenReversalLog> oxygenReversalLogs = oxygenReversalLogService.findByExampleSort(specification, null);
		List<OxygenReversalLogDto> oxygenReversalLogDtos = oxygenReversalLogs.stream()
					.map(oxygenReversalLog -> toLazyDto(oxygenReversalLog))
					.collect(Collectors.toList());
		return ResultGenerator.success(oxygenReversalLogDtos);
	}

	@ApiOperation(value = "氧反翘记录详细信息")
	@GetMapping("/{id}")
	public Result<OxygenReversalLogDto> get(@PathVariable @ApiParam(value="氧反翘记录id") Long id) {
		OxygenReversalLog oxygenReversalLog = oxygenReversalLogService.get(id);
		if (null == oxygenReversalLog){
			return ResultGenerator.fail("该氧反翘记录不存在");
		}
		return ResultGenerator.success(toEagerDto(oxygenReversalLog));
	}

	@ApiOperation(value = "创建氧反翘记录")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "氧反翘记录对象") OxygenReversalLogDto oxygenReversalLogDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		OxygenReversalLog oxygenReversalLog = fromDto(oxygenReversalLogDto);
		oxygenReversalLogService.saveOrUpdate(oxygenReversalLog);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "更新氧反翘记录")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "氧反翘记录") OxygenReversalLogDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors()){
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
        if (dto.getId() == null){
            return ResultGenerator.fail("id不能为空");
		}
		OxygenReversalLog entity = oxygenReversalLogService.get(dto.getId());
        if (entity == null){
            return ResultGenerator.fail("原信息不存在");
		}
        try {
            OxygenReversalLog entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			oxygenReversalLogService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除氧反翘记录")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
		for (Integer id : ids) {
				oxygenReversalLogService.delete(id);
		}
		return ResultGenerator.success();
	}

	public OxygenReversalLogDto toLazyDto(OxygenReversalLog entity) {
		return modelMapper.map(entity, OxygenReversalLogDto.class);
	}

	public OxygenReversalLogDto toEagerDto(OxygenReversalLog entity) {
		OxygenReversalLogDto dto = modelMapper.map(entity, OxygenReversalLogDto.class);
		return dto;
	}

	public OxygenReversalLog fromDto(OxygenReversalLogDto dto) {
		return modelMapper.map(dto, OxygenReversalLog.class);
	}


}
