package com.sw.mes.yb.deviceCheck.controller;

import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import com.sw.mes.yb.deviceCheck.dto.DeviceCheckTimeRecordDto;
import com.sw.mes.yb.deviceCheck.entity.DeviceCheckTimeRecord;
import com.sw.mes.yb.deviceCheck.service.DeviceCheckTimeRecordService;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;


/**
* @Author
* @Description：
* @Date: Create in
* @Modified By:
*/
@RestController
@RequestMapping("/deviceCheckTimeRecord")
@Slf4j
@Api(description = "圆棒-拆清点检时间接收表接口")
public class DeviceCheckTimeRecordController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private DeviceCheckTimeRecordService deviceCheckTimeRecordService;

	@ApiOperation(value = "分页查询圆棒-拆清点检时间接收表列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<DeviceCheckTimeRecordDto>> listByPage(ServletRequest request, @ApiIgnore RequestPage requestPage) {
		Specification<DeviceCheckTimeRecord> specification = DynamicSpecifications.bySearchFilter(request, DeviceCheckTimeRecord.class);
    	Page<DeviceCheckTimeRecord> page = deviceCheckTimeRecordService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<DeviceCheckTimeRecordDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有圆棒-拆清点检时间接收表")
	@GetMapping("/listAll")
	public Result<List<DeviceCheckTimeRecordDto>> listAll(ServletRequest request) {
		Specification<DeviceCheckTimeRecord> specification = DynamicSpecifications.bySearchFilter(request, DeviceCheckTimeRecord.class);
		List<DeviceCheckTimeRecord> deviceCheckTimeRecords = deviceCheckTimeRecordService.findByExampleSort(specification, null);
		List<DeviceCheckTimeRecordDto> deviceCheckTimeRecordDtos = deviceCheckTimeRecords.stream()
					.map(deviceCheckTimeRecord -> toLazyDto(deviceCheckTimeRecord))
					.collect(Collectors.toList());
		return ResultGenerator.success(deviceCheckTimeRecordDtos);
	}

	@ApiOperation(value = "圆棒-拆清点检时间接收表详细信息")
	@GetMapping("/{id}")
	public Result<DeviceCheckTimeRecordDto> get(@PathVariable @ApiParam(value="圆棒-拆清点检时间接收表id") Integer id) {
		DeviceCheckTimeRecord deviceCheckTimeRecord = deviceCheckTimeRecordService.get(id);
		if (null == deviceCheckTimeRecord)
			return ResultGenerator.fail("该圆棒-拆清点检时间接收表不存在");
		return ResultGenerator.success(toEagerDto(deviceCheckTimeRecord));
	}

	@ApiOperation(value = "创建圆棒-拆清点检时间接收表")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "圆棒-拆清点检时间接收表对象") DeviceCheckTimeRecordDto deviceCheckTimeRecordDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		DeviceCheckTimeRecord deviceCheckTimeRecord = fromDto(deviceCheckTimeRecordDto);
		deviceCheckTimeRecordService.saveOrUpdate(deviceCheckTimeRecord);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "更新圆棒-拆清点检时间接收表")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "圆棒-拆清点检时间接收表") DeviceCheckTimeRecordDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors())
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        if (dto.getId() == null)
            return ResultGenerator.fail("id不能为空");
            DeviceCheckTimeRecord entity = deviceCheckTimeRecordService.get(dto.getId());
        if (entity == null)
            return ResultGenerator.fail("原信息不存在");
        try {
            DeviceCheckTimeRecord entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			deviceCheckTimeRecordService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除圆棒-拆清点检时间接收表")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
		for (Integer id : ids) {
				deviceCheckTimeRecordService.delete(id);
		}
		return ResultGenerator.success();
	}

	public DeviceCheckTimeRecordDto toLazyDto(DeviceCheckTimeRecord entity) {
		return modelMapper.map(entity, DeviceCheckTimeRecordDto.class);
	}

	public DeviceCheckTimeRecordDto toEagerDto(DeviceCheckTimeRecord entity) {
		DeviceCheckTimeRecordDto dto = modelMapper.map(entity, DeviceCheckTimeRecordDto.class);
		return dto;
	}

	public DeviceCheckTimeRecord fromDto(DeviceCheckTimeRecordDto dto) {
		return modelMapper.map(dto, DeviceCheckTimeRecord.class);
	}


}
