package com.sw.mes.yb.deviceCheck.controller;

import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import com.sw.mes.yb.deviceCheck.dto.DeviceCleanRecordDto;
import com.sw.mes.yb.deviceCheck.entity.DeviceCleanRecord;
import com.sw.mes.yb.deviceCheck.service.DeviceCleanRecordService;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;


/**
* @Author
* @Description：
* @Date: Create in
* @Modified By:
*/
@RestController
@RequestMapping("/deviceCleanRecord")
@Slf4j
@Api(description = "圆棒-DE下发的拆清记录接口")
public class DeviceCleanRecordController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private DeviceCleanRecordService deviceCleanRecordService;

	@ApiOperation(value = "分页查询圆棒-DE下发的拆清记录列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<DeviceCleanRecordDto>> listByPage(ServletRequest request, @ApiIgnore RequestPage requestPage) {
		Specification<DeviceCleanRecord> specification = DynamicSpecifications.bySearchFilter(request, DeviceCleanRecord.class);
    	Page<DeviceCleanRecord> page = deviceCleanRecordService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<DeviceCleanRecordDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有圆棒-DE下发的拆清记录")
	@GetMapping("/listAll")
	public Result<List<DeviceCleanRecordDto>> listAll(ServletRequest request) {
		Specification<DeviceCleanRecord> specification = DynamicSpecifications.bySearchFilter(request, DeviceCleanRecord.class);
		List<DeviceCleanRecord> deviceCleanRecords = deviceCleanRecordService.findByExampleSort(specification, null);
		List<DeviceCleanRecordDto> deviceCleanRecordDtos = deviceCleanRecords.stream()
					.map(deviceCleanRecord -> toLazyDto(deviceCleanRecord))
					.collect(Collectors.toList());
		return ResultGenerator.success(deviceCleanRecordDtos);
	}

	@ApiOperation(value = "圆棒-DE下发的拆清记录详细信息")
	@GetMapping("/{id}")
	public Result<DeviceCleanRecordDto> get(@PathVariable @ApiParam(value="圆棒-DE下发的拆清记录id") Integer id) {
		DeviceCleanRecord deviceCleanRecord = deviceCleanRecordService.get(id);
		if (null == deviceCleanRecord)
			return ResultGenerator.fail("该圆棒-DE下发的拆清记录不存在");
		return ResultGenerator.success(toEagerDto(deviceCleanRecord));
	}

	@ApiOperation(value = "创建圆棒-DE下发的拆清记录")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "圆棒-DE下发的拆清记录对象") DeviceCleanRecordDto deviceCleanRecordDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		DeviceCleanRecord deviceCleanRecord = fromDto(deviceCleanRecordDto);
		deviceCleanRecordService.saveOrUpdate(deviceCleanRecord);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "更新圆棒-DE下发的拆清记录")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "圆棒-DE下发的拆清记录") DeviceCleanRecordDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors())
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        if (dto.getId() == null)
            return ResultGenerator.fail("id不能为空");
            DeviceCleanRecord entity = deviceCleanRecordService.get(dto.getId());
        if (entity == null)
            return ResultGenerator.fail("原信息不存在");
        try {
            DeviceCleanRecord entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			deviceCleanRecordService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除圆棒-DE下发的拆清记录")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
		for (Integer id : ids) {
				deviceCleanRecordService.delete(id);
		}
		return ResultGenerator.success();
	}

	public DeviceCleanRecordDto toLazyDto(DeviceCleanRecord entity) {
		return modelMapper.map(entity, DeviceCleanRecordDto.class);
	}

	public DeviceCleanRecordDto toEagerDto(DeviceCleanRecord entity) {
		DeviceCleanRecordDto dto = modelMapper.map(entity, DeviceCleanRecordDto.class);
		return dto;
	}

	public DeviceCleanRecord fromDto(DeviceCleanRecordDto dto) {
		return modelMapper.map(dto, DeviceCleanRecord.class);
	}


}
