package com.sw.mes.yb.mcs.controller;

import com.sw.mes.core.context.BaseContextHandler;
import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;
import com.sw.mes.yb.mcs.entity.RecycledMaterialCaseRecord;
import com.sw.mes.yb.mcs.dto.RecycledMaterialCaseRecordDto;
import com.sw.mes.yb.mcs.service.RecycledMaterialCaseRecordService;


/**
* @Author
* @Description：
* @Date: Create in
* @Modified By:
*/
@RestController
@RequestMapping("/recycledMaterialCaseRecord")
@Slf4j
@Api(description = "RecycledMaterialCaseRecord接口")
public class RecycledMaterialCaseRecordController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private RecycledMaterialCaseRecordService recycledMaterialCaseRecordService;

	@ApiOperation(value = "分页查询RecycledMaterialCaseRecord列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<RecycledMaterialCaseRecordDto>> listByPage(ServletRequest request,@ApiIgnore RequestPage requestPage) {
		Specification<RecycledMaterialCaseRecord> specification = DynamicSpecifications.bySearchFilter(request, RecycledMaterialCaseRecord.class);
    	Page<RecycledMaterialCaseRecord> page = recycledMaterialCaseRecordService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<RecycledMaterialCaseRecordDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有RecycledMaterialCaseRecord")
	@GetMapping("/listAll")
	public Result<List<RecycledMaterialCaseRecordDto>> listAll(ServletRequest request) {
		Specification<RecycledMaterialCaseRecord> specification = DynamicSpecifications.bySearchFilter(request, RecycledMaterialCaseRecord.class);
		List<RecycledMaterialCaseRecord> recycledMaterialCaseRecords = recycledMaterialCaseRecordService.findByExampleSort(specification, null);
		List<RecycledMaterialCaseRecordDto> recycledMaterialCaseRecordDtos = recycledMaterialCaseRecords.stream()
					.map(recycledMaterialCaseRecord -> toLazyDto(recycledMaterialCaseRecord))
					.collect(Collectors.toList());
		return ResultGenerator.success(recycledMaterialCaseRecordDtos);
	}

	@ApiOperation(value = "RecycledMaterialCaseRecord详细信息")
	@GetMapping("/{id}")
	public Result<RecycledMaterialCaseRecordDto> get(@PathVariable @ApiParam(value="RecycledMaterialCaseRecordid") Long id) {
		RecycledMaterialCaseRecord recycledMaterialCaseRecord = recycledMaterialCaseRecordService.get(id);
		if (null == recycledMaterialCaseRecord)
			return ResultGenerator.fail("该RecycledMaterialCaseRecord不存在");
		return ResultGenerator.success(toEagerDto(recycledMaterialCaseRecord));
	}

	@ApiOperation(value = "创建RecycledMaterialCaseRecord")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "RecycledMaterialCaseRecord对象") RecycledMaterialCaseRecordDto recycledMaterialCaseRecordDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		RecycledMaterialCaseRecord recycledMaterialCaseRecord = fromDto(recycledMaterialCaseRecordDto);
		recycledMaterialCaseRecordService.saveOrUpdate(recycledMaterialCaseRecord);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "更新RecycledMaterialCaseRecord")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "RecycledMaterialCaseRecord") RecycledMaterialCaseRecordDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors())
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        if (dto.getId() == null)
            return ResultGenerator.fail("id不能为空");
            RecycledMaterialCaseRecord entity = recycledMaterialCaseRecordService.get(dto.getId());
        if (entity == null)
            return ResultGenerator.fail("原信息不存在");
        try {
            RecycledMaterialCaseRecord entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			recycledMaterialCaseRecordService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除RecycledMaterialCaseRecord")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
		for (Integer id : ids) {
				recycledMaterialCaseRecordService.delete(id);
		}
		return ResultGenerator.success();
	}

	public RecycledMaterialCaseRecordDto toLazyDto(RecycledMaterialCaseRecord entity) {
		return modelMapper.map(entity, RecycledMaterialCaseRecordDto.class);
	}

	public RecycledMaterialCaseRecordDto toEagerDto(RecycledMaterialCaseRecord entity) {
		RecycledMaterialCaseRecordDto dto = modelMapper.map(entity, RecycledMaterialCaseRecordDto.class);
		return dto;
	}

	public RecycledMaterialCaseRecord fromDto(RecycledMaterialCaseRecordDto dto) {
		return modelMapper.map(dto, RecycledMaterialCaseRecord.class);
	}


}
