package com.sw.mes.yb.mcs.controller;

import com.sw.mes.core.context.BaseContextHandler;
import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;
import com.sw.mes.yb.mcs.entity.SegmentInspectionPassPointInfo;
import com.sw.mes.yb.mcs.dto.SegmentInspectionPassPointInfoDto;
import com.sw.mes.yb.mcs.service.SegmentInspectionPassPointInfoService;


/**
* @Author
* @Description：
* @Date: Create in
* @Modified By:
*/
@RestController
@RequestMapping("/segmentInspectionPassPointInfo")
@Slf4j
@Api(description = "SegmentInspectionPassPointInfo接口")
public class SegmentInspectionPassPointInfoController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private SegmentInspectionPassPointInfoService segmentInspectionPassPointInfoService;

	@ApiOperation(value = "分页查询SegmentInspectionPassPointInfo列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<SegmentInspectionPassPointInfoDto>> listByPage(ServletRequest request,@ApiIgnore RequestPage requestPage) {
		Specification<SegmentInspectionPassPointInfo> specification = DynamicSpecifications.bySearchFilter(request, SegmentInspectionPassPointInfo.class);
    	Page<SegmentInspectionPassPointInfo> page = segmentInspectionPassPointInfoService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<SegmentInspectionPassPointInfoDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有SegmentInspectionPassPointInfo")
	@GetMapping("/listAll")
	public Result<List<SegmentInspectionPassPointInfoDto>> listAll(ServletRequest request) {
		Specification<SegmentInspectionPassPointInfo> specification = DynamicSpecifications.bySearchFilter(request, SegmentInspectionPassPointInfo.class);
		List<SegmentInspectionPassPointInfo> segmentInspectionPassPointInfos = segmentInspectionPassPointInfoService.findByExampleSort(specification, null);
		List<SegmentInspectionPassPointInfoDto> segmentInspectionPassPointInfoDtos = segmentInspectionPassPointInfos.stream()
					.map(segmentInspectionPassPointInfo -> toLazyDto(segmentInspectionPassPointInfo))
					.collect(Collectors.toList());
		return ResultGenerator.success(segmentInspectionPassPointInfoDtos);
	}

	@ApiOperation(value = "SegmentInspectionPassPointInfo详细信息")
	@GetMapping("/{id}")
	public Result<SegmentInspectionPassPointInfoDto> get(@PathVariable @ApiParam(value="SegmentInspectionPassPointInfoid") Long id) {
		SegmentInspectionPassPointInfo segmentInspectionPassPointInfo = segmentInspectionPassPointInfoService.get(id);
		if (null == segmentInspectionPassPointInfo)
			return ResultGenerator.fail("该SegmentInspectionPassPointInfo不存在");
		return ResultGenerator.success(toEagerDto(segmentInspectionPassPointInfo));
	}

	@ApiOperation(value = "创建SegmentInspectionPassPointInfo")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "SegmentInspectionPassPointInfo对象") SegmentInspectionPassPointInfoDto segmentInspectionPassPointInfoDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		SegmentInspectionPassPointInfo segmentInspectionPassPointInfo = fromDto(segmentInspectionPassPointInfoDto);
		segmentInspectionPassPointInfoService.saveOrUpdate(segmentInspectionPassPointInfo);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "更新SegmentInspectionPassPointInfo")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "SegmentInspectionPassPointInfo") SegmentInspectionPassPointInfoDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors())
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        if (dto.getId() == null)
            return ResultGenerator.fail("id不能为空");
            SegmentInspectionPassPointInfo entity = segmentInspectionPassPointInfoService.get(dto.getId());
        if (entity == null)
            return ResultGenerator.fail("原信息不存在");
        try {
            SegmentInspectionPassPointInfo entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			segmentInspectionPassPointInfoService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除SegmentInspectionPassPointInfo")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
		for (Integer id : ids) {
				segmentInspectionPassPointInfoService.delete(id);
		}
		return ResultGenerator.success();
	}

	public SegmentInspectionPassPointInfoDto toLazyDto(SegmentInspectionPassPointInfo entity) {
		return modelMapper.map(entity, SegmentInspectionPassPointInfoDto.class);
	}

	public SegmentInspectionPassPointInfoDto toEagerDto(SegmentInspectionPassPointInfo entity) {
		SegmentInspectionPassPointInfoDto dto = modelMapper.map(entity, SegmentInspectionPassPointInfoDto.class);
		return dto;
	}

	public SegmentInspectionPassPointInfo fromDto(SegmentInspectionPassPointInfoDto dto) {
		return modelMapper.map(dto, SegmentInspectionPassPointInfo.class);
	}


}
