package com.sw.mes.yb.mcs.dto;

import com.fasterxml.jackson.annotation.JsonProperty;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import java.math.BigDecimal;

/**
 * 短棒检验信息
 *
 * @Author: GaoXinJian
 * @Date: 2022-01-13 14:10
 */
@ApiModel(value = "圆棒检测结果上传Dto")
@Data
public class YbCheckResultDto {

    @ApiModelProperty(value = "电阻率设备编号")
    private String resistivityDeviceNo;

    @ApiModelProperty(value = "少子寿命设备编号")
    private String lifeDeviceNo;

    @ApiModelProperty(value = "晶棒编号")
    private String sectionNo;

    @ApiModelProperty(value = "导电型号(P,N,P/N)")
    private String conductiveType;

    @ApiModelProperty(value = "圆棒长度mm")
    private BigDecimal actualLength;

    @ApiModelProperty(value = "重量")
    private BigDecimal actualWeight;

    @ApiModelProperty(value = "直径Min")
    private BigDecimal diameterMin;

    @ApiModelProperty(value = "直径Max")
    private BigDecimal diameterMax;

    @ApiModelProperty(value = "规格")
    private String productTypeSecond;

    @ApiModelProperty(value = "细总长")
    private BigDecimal fineDiameter;

    @ApiModelProperty(value = "细减长")
    private BigDecimal fineCutLength;

    @ApiModelProperty(value = "头斜")
    private BigDecimal headSlope;

    @ApiModelProperty(value = "尾斜")
    private BigDecimal tailSlope;

    @ApiModelProperty(value = "孪晶长度")
    private BigDecimal twinLength;

    @ApiModelProperty(value = "裂纹")
    private BigDecimal crack;

    @ApiModelProperty(value = "崩边长度")
    private BigDecimal edgeBark;

    @ApiModelProperty(value = "崩晶线")
    BigDecimal collapseLine;

    @ApiModelProperty(value = "晶线分叉【0：默认，否；1：是；】")
    private Integer isFork;

    // 头斜或尾斜有值就是，没有否
//    @ApiModelProperty(value = "斜【0：默认，否；1：是；】" )
//    Integer isBevel;
    @ApiModelProperty(value = "扭")
    private BigDecimal distortion;

    @ApiModelProperty(value = "头段A电阻率均值")
    @JsonProperty("tRadialCentre")
    private BigDecimal tRadialCentre;

    @ApiModelProperty(value = "头段B电阻率均值")
    @JsonProperty("tRadialEdge1")
    private BigDecimal tRadialEdge1;

    @ApiModelProperty(value = "尾段A电阻率均值")
    @JsonProperty("wRadialCentre")
    private BigDecimal wRadialCentre;

    @ApiModelProperty(value = "尾段B电阻率均值")
    @JsonProperty("wRadialEdge1")
    private BigDecimal wRadialEdge1;

    @ApiModelProperty(value = "头段A少子寿命均值")
    @JsonProperty("tCentreLife")
    private BigDecimal tCentreLife;

    @ApiModelProperty(value = "头段B少子寿命均值")
    @JsonProperty("tLifeEdge1")
    private BigDecimal tLifeEdge1;

    @ApiModelProperty(value = "尾段A少子寿命均值")
    @JsonProperty("wCentreLife")
    private BigDecimal wCentreLife;

    @ApiModelProperty(value = "尾段B少子寿命均值")
    @JsonProperty("wLifeEdge1")
    private BigDecimal wLifeEdge1;

    @ApiModelProperty(value = "径向电阻率T边缘2")
    @JsonProperty("tRadialEdge2")
    private BigDecimal tRadialEdge2;

    @ApiModelProperty(value = "径向电阻率T边缘3")
    @JsonProperty("tRadialEdge3")
    private BigDecimal tRadialEdge3;

    @ApiModelProperty(value = "径向电阻率T边缘4")
    @JsonProperty("tRadialEdge4")
    private BigDecimal tRadialEdge4;

    @ApiModelProperty(value = "径向电阻率W边缘2")
    @JsonProperty("wRadialEdge2")
    private BigDecimal wRadialEdge2;

    @ApiModelProperty(value = "径向电阻率W边缘3")
    @JsonProperty("wRadialEdge3")
    private BigDecimal wRadialEdge3;

    @ApiModelProperty(value = "径向电阻率W边缘4")
    @JsonProperty("wRadialEdge4")
    private BigDecimal wRadialEdge4;

    @ApiModelProperty(value = "T边缘寿命2")
    @JsonProperty("tLifeEdge2")
    private BigDecimal tLifeEdge2;

    @ApiModelProperty(value = "T边缘寿命3")
    @JsonProperty("tLifeEdge3")
    private BigDecimal tLifeEdge3;

    @ApiModelProperty(value = "T边缘寿命4")
    @JsonProperty("tLifeEdge4")
    private BigDecimal tLifeEdge4;

    @ApiModelProperty(value = "T边缘寿命5")
    @JsonProperty("tLifeEdge5")
    private BigDecimal tLifeEdge5;

    @ApiModelProperty(value = "T边缘寿命6")
    @JsonProperty("tLifeEdge6")
    private BigDecimal tLifeEdge6;

    @ApiModelProperty(value = "T边缘寿命7")
    private BigDecimal tLifeEdge7;

    @ApiModelProperty(value = "T边缘寿命8")
    private BigDecimal tLifeEdge8;

    @ApiModelProperty(value = "W边缘寿命2")
    @JsonProperty("wLifeEdge2")
    private BigDecimal wLifeEdge2;

    @ApiModelProperty(value = "W边缘寿命3")
    @JsonProperty("wLifeEdge3")
    private BigDecimal wLifeEdge3;

    @ApiModelProperty(value = "W边缘寿命4")
    @JsonProperty("wLifeEdge4")
    private BigDecimal wLifeEdge4;

    @ApiModelProperty(value = "W边缘寿命5")
    @JsonProperty("wLifeEdge5")
    private BigDecimal wLifeEdge5;

    @ApiModelProperty(value = "W边缘寿命6")
    @JsonProperty("wLifeEdge6")
    private BigDecimal wLifeEdge6;

    @ApiModelProperty(value = "W边缘寿命7")
    private BigDecimal wLifeEdge7;

    @ApiModelProperty(value = "W边缘寿命8")
    private BigDecimal wLifeEdge8;

    @ApiModelProperty(value = "检验人员：设备自检")
    private String inspector;

    @ApiModelProperty(value = "托盘号")
    private String code;

    @ApiModelProperty(value = "温度")
    private BigDecimal tempValue;

    @ApiModelProperty(value = "平均直径")
    private BigDecimal diameterAvg;

    @ApiModelProperty(value = "检验线编号")
    private String lineNo;

    @ApiModelProperty(value = "头温度")
    private String temperature;

    @ApiModelProperty(value = "尾温度")
    private String temperatureEnd;

    @ApiModelProperty(value = "外观检验反切")
    private String checkReback;
}