package com.sw.mes.yb.pad.controller;

import com.sw.mes.core.dto.mes.pc.yb.SectionNoDetailDto;
import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;

import com.sw.mes.yb.pad.entity.SectionNoDetail;
import com.sw.mes.yb.pad.dto.PieceSectionNoDetailDto;
import com.sw.mes.yb.pad.service.SectionNoDetailService;


/**
 * @Author
 * @Description：
 * @Date: Create in
 * @Modified By:
 */
@RestController
@RequestMapping("/SectionNoDetail")
@Slf4j
@Api(description = "晶段信息接口")
public class SectionNoDetailController {
    @Autowired
    ModelMapper modelMapper;

    @Autowired
    private SectionNoDetailService sectionNoDetailService;

    @ApiOperation(value = "分页查询晶段信息列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
            @ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
    })
    @GetMapping("/listByPage")
    public Result<ResponsePage<PieceSectionNoDetailDto>> listByPage(ServletRequest request, @ApiIgnore RequestPage requestPage) {
        Specification<SectionNoDetail> specification = DynamicSpecifications.bySearchFilter(request, SectionNoDetail.class);
        Page<SectionNoDetail> page = sectionNoDetailService.findByExample(specification, PageUtil.createPageable(requestPage));
        List<PieceSectionNoDetailDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
        ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
    }

    @ApiOperation(value = "获取所有晶段信息")
    @GetMapping("/listAll")
    public Result<List<PieceSectionNoDetailDto>> listAll(ServletRequest request) {
        Specification<SectionNoDetail> specification = DynamicSpecifications.bySearchFilter(request, SectionNoDetail.class);
        List<SectionNoDetail> sectionNoDetails = sectionNoDetailService.findByExampleSort(specification, null);
        List<PieceSectionNoDetailDto> pieceSectionNoDetailDtos = sectionNoDetails.stream()
                .map(pieceSectionNoDetail -> toLazyDto(pieceSectionNoDetail))
                .collect(Collectors.toList());
        return ResultGenerator.success(pieceSectionNoDetailDtos);
    }

    @ApiOperation(value = "晶段信息详细信息")
    @GetMapping("/{id}")
    public Result<PieceSectionNoDetailDto> get(@PathVariable @ApiParam(value = "PieceSectionNoDetailid") Long id) {
        SectionNoDetail sectionNoDetail = sectionNoDetailService.get(id);
        if (null == sectionNoDetail)
            return ResultGenerator.fail("该PieceSectionNoDetail不存在");
        return ResultGenerator.success(toEagerDto(sectionNoDetail));
    }

    @ApiOperation(value = "创建晶段信息")
    @PostMapping("/create")
    public Result create(@RequestBody @Valid @ApiParam(value = "PieceSectionNoDetail对象") PieceSectionNoDetailDto pieceSectionNoDetailDto, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        }
        SectionNoDetail sectionNoDetail = fromDto(pieceSectionNoDetailDto);
        sectionNoDetailService.saveOrUpdate(sectionNoDetail);
        return ResultGenerator.success();
    }

    @ApiOperation(value = "更新晶段信息")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "PieceSectionNoDetail") PieceSectionNoDetailDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors())
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        if (dto.getId() == null)
            return ResultGenerator.fail("id不能为空");
        SectionNoDetail entity = sectionNoDetailService.get(dto.getId());
        if (entity == null)
            return ResultGenerator.fail("原信息不存在");
        try {
            SectionNoDetail entityfromFto = fromDto(dto);
            BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
            sectionNoDetailService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

    @ApiOperation(value = "删除晶段信息")
    @PostMapping("/delete")
    public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
        for (Integer id : ids) {
            sectionNoDetailService.delete(id);
        }
        return ResultGenerator.success();
    }

    @ApiOperation(value = "根据晶段编号获取信息")
    @GetMapping("/getSectionNoDetailDtoBySectionNo")
    public Result<SectionNoDetailDto> getSectionNoDetailDtoBySectionNo(@RequestParam(value = "sectionNo") String sectionNo) {
        SectionNoDetail sectionNoDetail = sectionNoDetailService.findBySectionNo(sectionNo);
        SectionNoDetailDto sectionNoDetailDto = modelMapper.map(sectionNoDetail, SectionNoDetailDto.class);
        return ResultGenerator.success(sectionNoDetailDto);
    }

    public PieceSectionNoDetailDto toLazyDto(SectionNoDetail entity) {
        return modelMapper.map(entity, PieceSectionNoDetailDto.class);
    }

    public PieceSectionNoDetailDto toEagerDto(SectionNoDetail entity) {
        PieceSectionNoDetailDto dto = modelMapper.map(entity, PieceSectionNoDetailDto.class);
        return dto;
    }

    public SectionNoDetail fromDto(PieceSectionNoDetailDto dto) {
        return modelMapper.map(dto, SectionNoDetail.class);
    }


}
