package com.sw.mes.yb.pc.controller;

import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import com.sw.mes.yb.pc.dto.QualityPrintLogDto;
import com.sw.mes.yb.pc.entity.QualityPrintLog;
import com.sw.mes.yb.pc.service.QualityPrintLogService;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;


/**
* @Author
* @Description：
* @Date: Create in
* @Modified By:
*/
@RestController
@RequestMapping("/qualityPrintLog")
@Slf4j
@Api(description = "合格证打印记录接口")
public class QualityPrintLogController {
	@Autowired
    ModelMapper modelMapper;

	@Autowired
	private QualityPrintLogService qualityPrintLogService;

	@ApiOperation(value = "分页查询合格证打印记录列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<QualityPrintLogDto>> listByPage(ServletRequest request, @ApiIgnore RequestPage requestPage) {
		Specification<QualityPrintLog> specification = DynamicSpecifications.bySearchFilter(request, QualityPrintLog.class);
    	Page<QualityPrintLog> page = qualityPrintLogService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<QualityPrintLogDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有合格证打印记录")
	@GetMapping("/listAll")
	public Result<List<QualityPrintLogDto>> listAll(ServletRequest request) {
		Specification<QualityPrintLog> specification = DynamicSpecifications.bySearchFilter(request, QualityPrintLog.class);
		List<QualityPrintLog> qualityPrintLogs = qualityPrintLogService.findByExampleSort(specification, null);
		List<QualityPrintLogDto> qualityPrintLogDtos = qualityPrintLogs.stream()
					.map(qualityPrintLog -> toLazyDto(qualityPrintLog))
					.collect(Collectors.toList());
		return ResultGenerator.success(qualityPrintLogDtos);
	}

	@ApiOperation(value = "合格证打印记录详细信息")
	@GetMapping("/{id}")
	public Result<QualityPrintLogDto> get(@PathVariable @ApiParam(value="合格证打印记录id") Integer id) {
		QualityPrintLog qualityPrintLog = qualityPrintLogService.get(id);
		if (null == qualityPrintLog) {
            return ResultGenerator.fail("该合格证打印记录不存在");
        }
		return ResultGenerator.success(toEagerDto(qualityPrintLog));
	}

	@ApiOperation(value = "创建合格证打印记录")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "合格证打印记录对象") QualityPrintLogDto qualityPrintLogDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		QualityPrintLog qualityPrintLog = fromDto(qualityPrintLogDto);
		qualityPrintLogService.saveOrUpdate(qualityPrintLog);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "更新合格证打印记录")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "合格证打印记录") QualityPrintLogDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        }
        if (dto.getId() == null) {
            return ResultGenerator.fail("id不能为空");
        }
            QualityPrintLog entity = qualityPrintLogService.get(dto.getId());
        if (entity == null) {
            return ResultGenerator.fail("原信息不存在");
        }
        try {
            QualityPrintLog entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			qualityPrintLogService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除合格证打印记录")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
		for (Integer id : ids) {
				qualityPrintLogService.delete(id);
		}
		return ResultGenerator.success();
	}

	public QualityPrintLogDto toLazyDto(QualityPrintLog entity) {
		return modelMapper.map(entity, QualityPrintLogDto.class);
	}

	public QualityPrintLogDto toEagerDto(QualityPrintLog entity) {
		QualityPrintLogDto dto = modelMapper.map(entity, QualityPrintLogDto.class);
		return dto;
	}

	public QualityPrintLog fromDto(QualityPrintLogDto dto) {
		return modelMapper.map(dto, QualityPrintLog.class);
	}


}
