package com.sw.mes.yb.production.controller;

import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;
import com.sw.mes.yb.production.entity.BlastFurnaceDetailLog;
import com.sw.mes.yb.production.dto.BlastFurnaceDetail.BlastFurnaceDetailLogDto;
import com.sw.mes.yb.production.service.BlastFurnaceDetailLogService;


/**
* @Author
* @Description：
* @Date: Create in
* @Modified By:
*/
@RestController
@RequestMapping("/blastFurnaceDetailLog")
@Slf4j
@Api(description = "装料(配料)明细记录表接口")
public class BlastFurnaceDetailLogController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private BlastFurnaceDetailLogService blastFurnaceDetailLogService;

	@ApiOperation(value = "分页查询装料(配料)明细记录表列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<BlastFurnaceDetailLogDto>> listByPage(ServletRequest request,@ApiIgnore RequestPage requestPage) {
		Specification<BlastFurnaceDetailLog> specification = DynamicSpecifications.bySearchFilter(request, BlastFurnaceDetailLog.class);
    	Page<BlastFurnaceDetailLog> page = blastFurnaceDetailLogService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<BlastFurnaceDetailLogDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有装料(配料)明细记录表")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "search_EQ_blastFurnaceId", value = "装料任务记录Id (LoadingTask.id)", dataType = "Long", paramType = "query"),
	})
	@ApiParam(name = "search_EQ_blastFurnaceId",value = "装料任务Id (LoadingTask.id)",type = "Long",required = true)
	@GetMapping("/listAll")
	public Result<List<BlastFurnaceDetailLogDto>> listAll(ServletRequest request) {
		Specification<BlastFurnaceDetailLog> specification = DynamicSpecifications.bySearchFilter(request, BlastFurnaceDetailLog.class);
		List<BlastFurnaceDetailLog> blastFurnaceDetailLogs = blastFurnaceDetailLogService.findByExampleSort(specification, null);
		List<BlastFurnaceDetailLogDto> blastFurnaceDetailLogDtos = blastFurnaceDetailLogs.stream()
					.map(blastFurnaceDetailLog -> toLazyDto(blastFurnaceDetailLog))
					.collect(Collectors.toList());
		return ResultGenerator.success(blastFurnaceDetailLogDtos);
	}

	@ApiOperation(value = "装料(配料)明细记录表详细信息")
	@GetMapping("/{id}")
	public Result<BlastFurnaceDetailLogDto> get(@PathVariable @ApiParam(value="BlastFurnaceDetailLogid") Long id) {
		BlastFurnaceDetailLog blastFurnaceDetailLog = blastFurnaceDetailLogService.get(id);
		if (null == blastFurnaceDetailLog)
			return ResultGenerator.fail("该BlastFurnaceDetailLog不存在");
		return ResultGenerator.success(toEagerDto(blastFurnaceDetailLog));
	}

	@ApiOperation(value = "创建装料(配料)明细记录表")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "BlastFurnaceDetailLog对象") BlastFurnaceDetailLogDto blastFurnaceDetailLogDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		BlastFurnaceDetailLog blastFurnaceDetailLog = fromDto(blastFurnaceDetailLogDto);
		blastFurnaceDetailLogService.saveOrUpdate(blastFurnaceDetailLog);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "更新装料(配料)明细记录表")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "BlastFurnaceDetailLog") BlastFurnaceDetailLogDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors())
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        if (dto.getId() == null)
            return ResultGenerator.fail("id不能为空");
            BlastFurnaceDetailLog entity = blastFurnaceDetailLogService.get(dto.getId());
        if (entity == null)
            return ResultGenerator.fail("原信息不存在");
        try {
            BlastFurnaceDetailLog entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			blastFurnaceDetailLogService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除装料(配料)明细记录表")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
		for (Integer id : ids) {
				blastFurnaceDetailLogService.delete(id);
		}
		return ResultGenerator.success();
	}

	public BlastFurnaceDetailLogDto toLazyDto(BlastFurnaceDetailLog entity) {
		return modelMapper.map(entity, BlastFurnaceDetailLogDto.class);
	}

	public BlastFurnaceDetailLogDto toEagerDto(BlastFurnaceDetailLog entity) {
		BlastFurnaceDetailLogDto dto = modelMapper.map(entity, BlastFurnaceDetailLogDto.class);
		return dto;
	}

	public BlastFurnaceDetailLog fromDto(BlastFurnaceDetailLogDto dto) {
		return modelMapper.map(dto, BlastFurnaceDetailLog.class);
	}


}
