package com.sw.mes.yb.production.controller;

import com.sw.mes.core.context.BaseContextHandler;
import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;
import com.sw.mes.yb.production.entity.EquipmentClickLog;
import com.sw.mes.yb.production.dto.EquipmentClickLogDto;
import com.sw.mes.yb.production.service.EquipmentClickLogService;


/**
* @Author
* @Description：
* @Date: Create in
* @Modified By:
*/
@RestController
@RequestMapping("/productionEquipmentClickLog")
@Slf4j
@Api(description = "工控机操作记录接口")
public class EquipmentClickLogController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private EquipmentClickLogService equipmentClickLogService;

	@ApiOperation(value = "分页查询工控机操作记录列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<EquipmentClickLogDto>> listByPage(ServletRequest request,@ApiIgnore RequestPage requestPage) {
		Specification<EquipmentClickLog> specification = DynamicSpecifications.bySearchFilter(request, EquipmentClickLog.class);
    	Page<EquipmentClickLog> page = equipmentClickLogService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<EquipmentClickLogDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有工控机操作记录")
	@GetMapping("/listAll")
	public Result<List<EquipmentClickLogDto>> listAll(ServletRequest request) {
		Specification<EquipmentClickLog> specification = DynamicSpecifications.bySearchFilter(request, EquipmentClickLog.class);
		List<EquipmentClickLog> equipmentClickLogs = equipmentClickLogService.findByExampleSort(specification, null);
		List<EquipmentClickLogDto> equipmentClickLogDtos = equipmentClickLogs.stream()
					.map(equipmentClickLog -> toLazyDto(equipmentClickLog))
					.collect(Collectors.toList());
		return ResultGenerator.success(equipmentClickLogDtos);
	}

	@ApiOperation(value = "工控机操作记录详细信息")
	@GetMapping("/{id}")
	public Result<EquipmentClickLogDto> get(@PathVariable @ApiParam(value="工控机操作记录id") Long id) {
		EquipmentClickLog equipmentClickLog = equipmentClickLogService.get(id);
		if (null == equipmentClickLog) {
            return ResultGenerator.fail("该工控机操作记录不存在");
        }
		return ResultGenerator.success(toEagerDto(equipmentClickLog));
	}

	@ApiOperation(value = "创建工控机操作记录")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "工控机操作记录对象") EquipmentClickLogDto equipmentClickLogDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		EquipmentClickLog equipmentClickLog = fromDto(equipmentClickLogDto);
		equipmentClickLogService.saveOrUpdate(equipmentClickLog);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "更新工控机操作记录")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "工控机操作记录") EquipmentClickLogDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        }
        if (dto.getId() == null) {
            return ResultGenerator.fail("id不能为空");
        }
            EquipmentClickLog entity = equipmentClickLogService.get(dto.getId());
        if (entity == null) {
            return ResultGenerator.fail("原信息不存在");
        }
        try {
            EquipmentClickLog entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			equipmentClickLogService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除工控机操作记录")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Long[] ids) {
		for (Long id : ids) {
				equipmentClickLogService.delete(id);
		}
		return ResultGenerator.success();
	}

	public EquipmentClickLogDto toLazyDto(EquipmentClickLog entity) {
		return modelMapper.map(entity, EquipmentClickLogDto.class);
	}

	public EquipmentClickLogDto toEagerDto(EquipmentClickLog entity) {
		EquipmentClickLogDto dto = modelMapper.map(entity, EquipmentClickLogDto.class);
		return dto;
	}

	public EquipmentClickLog fromDto(EquipmentClickLogDto dto) {
		return modelMapper.map(dto, EquipmentClickLog.class);
	}


}
