package com.sw.mes.yb.production.controller;

import com.sw.mes.core.context.BaseContextHandler;
import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;
import com.sw.mes.yb.production.entity.ProcessWorkingHours;
import com.sw.mes.yb.production.dto.ProcessWorkingHoursDto;
import com.sw.mes.yb.production.service.ProcessWorkingHoursService;


/**
* @Author
* @Description：
* @Date: Create in
* @Modified By:
*/
@RestController
@RequestMapping("/ProcessWorkingHours")
@Slf4j
@Api(description = "ProcessWorkingHours接口")
public class ProcessWorkingHoursController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private ProcessWorkingHoursService processWorkingHoursService;

	@ApiOperation(value = "分页查询ProcessWorkingHours列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<ProcessWorkingHoursDto>> listByPage(ServletRequest request,@ApiIgnore RequestPage requestPage) {
		Specification<ProcessWorkingHours> specification = DynamicSpecifications.bySearchFilter(request, ProcessWorkingHours.class);
    	Page<ProcessWorkingHours> page = processWorkingHoursService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<ProcessWorkingHoursDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有ProcessWorkingHours")
	@GetMapping("/listAll")
	public Result<List<ProcessWorkingHoursDto>> listAll(ServletRequest request) {
		Specification<ProcessWorkingHours> specification = DynamicSpecifications.bySearchFilter(request, ProcessWorkingHours.class);
		List<ProcessWorkingHours> processWorkingHourss = processWorkingHoursService.findByExampleSort(specification, null);
		List<ProcessWorkingHoursDto> processWorkingHoursDtos = processWorkingHourss.stream()
					.map(processWorkingHours -> toLazyDto(processWorkingHours))
					.collect(Collectors.toList());
		return ResultGenerator.success(processWorkingHoursDtos);
	}

	@ApiOperation(value = "ProcessWorkingHours详细信息")
	@GetMapping("/{id}")
	public Result<ProcessWorkingHoursDto> get(@PathVariable @ApiParam(value="ProcessWorkingHoursid") Long id) {
		ProcessWorkingHours processWorkingHours = processWorkingHoursService.get(id);
		if (null == processWorkingHours)
			return ResultGenerator.fail("该ProcessWorkingHours不存在");
		return ResultGenerator.success(toEagerDto(processWorkingHours));
	}

	@ApiOperation(value = "创建ProcessWorkingHours")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "ProcessWorkingHours对象") ProcessWorkingHoursDto processWorkingHoursDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		ProcessWorkingHours processWorkingHours = fromDto(processWorkingHoursDto);
		processWorkingHoursService.saveOrUpdate(processWorkingHours);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "更新ProcessWorkingHours")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "ProcessWorkingHours") ProcessWorkingHoursDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors())
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        if (dto.getId() == null)
            return ResultGenerator.fail("id不能为空");
            ProcessWorkingHours entity = processWorkingHoursService.get(dto.getId());
        if (entity == null)
            return ResultGenerator.fail("原信息不存在");
        try {
            ProcessWorkingHours entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			processWorkingHoursService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除ProcessWorkingHours")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
		for (Integer id : ids) {
				processWorkingHoursService.delete(id);
		}
		return ResultGenerator.success();
	}

	public ProcessWorkingHoursDto toLazyDto(ProcessWorkingHours entity) {
		return modelMapper.map(entity, ProcessWorkingHoursDto.class);
	}

	public ProcessWorkingHoursDto toEagerDto(ProcessWorkingHours entity) {
		ProcessWorkingHoursDto dto = modelMapper.map(entity, ProcessWorkingHoursDto.class);
		return dto;
	}

	public ProcessWorkingHours fromDto(ProcessWorkingHoursDto dto) {
		return modelMapper.map(dto, ProcessWorkingHours.class);
	}


}
