package com.sw.mes.yb.production.controller;


import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;
import com.sw.mes.yb.production.entity.ProcessWorkingStep;
import com.sw.mes.yb.production.dto.ProcessWorkingStepDto;
import com.sw.mes.yb.production.service.ProcessWorkingStepService;


/**
 * @Author
 * @Description：
 * @Date: Create in
 * @Modified By:
 */
@RestController
@RequestMapping("/ProcessWorkingStep")
@Slf4j
@Api(description = "工艺工序接口")
public class ProcessWorkingStepController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private ProcessWorkingStepService processWorkingStepService;

	@ApiOperation(value = "分页查询工艺工序列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<ProcessWorkingStepDto>> listByPage(ServletRequest request,@ApiIgnore RequestPage requestPage) {
		Specification<ProcessWorkingStep> specification = DynamicSpecifications.bySearchFilter(request, ProcessWorkingStep.class);
    	Page<ProcessWorkingStep> page = processWorkingStepService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<ProcessWorkingStepDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有工艺工序")
	@GetMapping("/listAll")
	public Result<List<ProcessWorkingStepDto>> listAll(ServletRequest request) {
		Specification<ProcessWorkingStep> specification = DynamicSpecifications.bySearchFilter(request, ProcessWorkingStep.class);
		List<ProcessWorkingStep> processWorkingSteps = processWorkingStepService.findByExampleSort(specification, null);
		List<ProcessWorkingStepDto> processWorkingStepDtos = processWorkingSteps.stream()
					.map(processWorkingStep -> toLazyDto(processWorkingStep))
					.collect(Collectors.toList());
		return ResultGenerator.success(processWorkingStepDtos);
	}

	@ApiOperation(value = "工艺工序详细信息")
	@GetMapping("/{id}")
	public Result<ProcessWorkingStepDto> get(@PathVariable @ApiParam(value="工艺工序id") Long id) {
		ProcessWorkingStep processWorkingStep = processWorkingStepService.get(id);
		if (null == processWorkingStep){
			return ResultGenerator.fail("该工艺工序不存在");
		}
		return ResultGenerator.success(toEagerDto(processWorkingStep));
	}

	@ApiOperation(value = "创建工艺工序")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "工艺工序对象") ProcessWorkingStepDto processWorkingStepDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		ProcessWorkingStep processWorkingStep = fromDto(processWorkingStepDto);
		processWorkingStepService.saveOrUpdate(processWorkingStep);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "更新工艺工序")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "工艺工序") ProcessWorkingStepDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors()){
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
        if (dto.getId() == null){
            return ResultGenerator.fail("id不能为空");
		}
		ProcessWorkingStep entity = processWorkingStepService.get(dto.getId());
        if (entity == null){
            return ResultGenerator.fail("原信息不存在");
		}
        try {
            ProcessWorkingStep entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			processWorkingStepService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除工艺工序")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
		for (Integer id : ids) {
				processWorkingStepService.delete(id);
		}
		return ResultGenerator.success();
	}

	public ProcessWorkingStepDto toLazyDto(ProcessWorkingStep entity) {
		return modelMapper.map(entity, ProcessWorkingStepDto.class);
	}

	public ProcessWorkingStepDto toEagerDto(ProcessWorkingStep entity) {
		ProcessWorkingStepDto dto = modelMapper.map(entity, ProcessWorkingStepDto.class);
		return dto;
	}

	public ProcessWorkingStep fromDto(ProcessWorkingStepDto dto) {
		return modelMapper.map(dto, ProcessWorkingStep.class);
	}


}
