/**
 * There are <a href="https://github.com/ketayao/keta-custom">keta-custom</a> code generation
 */
package com.sw.mes.yb.production.entity;

import java.util.Date;
import java.math.BigDecimal;

import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import javax.persistence.*;
import org.hibernate.annotations.DynamicInsert;
import org.hibernate.annotations.DynamicUpdate;

/**
* @Author 李儒
* @Description：
* @Date: Create in
* @Modified By:
*/
@Data
@Entity
@Table(name="production_input_furnace_burden")
@DynamicUpdate
@DynamicInsert
public class InputFurnaceBurden{
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
    private Long id;
    
	/**
	 * 设备号
	 */
    @Column(length=100)
    private String equipmentCode;
    
	/**
	 * 炉次号
	 */
    @Column(length=100)
    private String lotNo;


	/**
	 * 下次复投是否完结【0：不是; 1 是;】
	 */
	@Column(nullable=false, length=3)
	private Integer isNextEnd;



	/**
	 * 所属系列
	 */
    @Column(length=50)
    private String series;
    
	/**
	 * 所属模块
	 */
    @Column(length=50)
    private String area;
    
	/**
	 * 叫料类型【0:默认，初始装料;】
	 */
    @Column(nullable=false, length=3)
    private Integer rawType;
    
	/**
	 * 载具编号
	 */
    @Column(length=100)
    private String vehicleCode;
    
	/**
	 * 灌料桶数
	 */
    @Column(length=255)
    private Integer pailNum;
    
	/**
	 * 灌料重量
	 */
    @Column(length=19)
    private BigDecimal weigth;
    
	/**
	 * 灌料人
	 */
    @Column(length=50)
    private String userName;
    
	/**
	 * 班次
	 */
    @Column(length=50)
    private String classNo;
    
	/**
	 * 班组
	 */
    @Column(length=255)
    private String teamGroup;
    
	/**
	 * 开始灌料时间
	 */
    @Column(nullable=false, length=19)
	@Temporal(TemporalType.TIMESTAMP)
    private Date startTime;
    
	/**
	 * 结束灌料时间
	 */
    @Column(length=19)
	@Temporal(TemporalType.TIMESTAMP)
    private Date endTime;
    
	/**
	 * 炉台类型【0:默认；0-量产;1-在建;2-研发;】
	 */
    @Column(nullable=false, length=3)
    private Integer furnaceType;
    
	/**
	 * SAP订单号
	 */
    @Column(length=100)
    private String sapNo;
    
	/**
	 * 料号
	 */
    @Column(length=100)
    private String productCode;
    
	/**
	 * 产品等级
	 */
    @Column(length=50)
    private String productGrade;
    
	/**
	 * 规格
	 */
    @Column(length=50)
    private String specs;
    
	/**
	 * 掺杂剂
	 */
    @Column(length=10)
    private String dopant;
    
	/**
	 * 产品类别
	 */
    @Column(length=50)
    private String productType;
    
	/**
	 * 左桶编号
	 */
    @Column(length=60)
    private String leftCode;

	/**
	 * 左桶灌料重量
	 */
	@Column(length=19)
	private BigDecimal leftWeight;

	/**
	 * 左桶退料(0:否, 1:是)
	 */
	@Column(length=4)
	private Integer leftBack;
    
	/**
	 * 右桶编号
	 */
    @Column(length=60)
    private String rightCode;

	/**
	 * 右桶灌料重量
	 */
	@Column(length=19)
	private BigDecimal rightWeight;

	/**
	 * 右桶退料(0:否, 1:是)
	 */
	@Column(length=4)
	private Integer rightBack;
    
	/**
	 * 备注
	 */
    @Column(length=255)
    private String remark;
    
	/**
	 * 灌料任务id(production_again_task_detail.id)
	 */
    @Column(length=19)
    private Long againTaskDetailId;


	/**
	 * 复投次数
	 */
	@Column(length=19)
	Integer againNum;

	/**
	 * 实际复投重量
	 */
	@Column(length=19)
	private BigDecimal againRealWeight;

	/**
	 * 确认投料(0:未确认, 1:已确认)
	 */
	@Column(length=4)
	private Integer confirm = 0;

	/**
	 * 确认投料时间
	 */
	@Column(nullable=false, length=19)
	@Temporal(TemporalType.TIMESTAMP)
	private Date confirmTime;
    

	public InputFurnaceBurden() {
	}

	public InputFurnaceBurden(Long id) {
		this.id = id;
	}
}
