package com.sw.mes.yb.report.controller;

import com.sw.mes.core.context.BaseContextHandler;
import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.*;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;
import com.sw.mes.yb.report.entity.BrokenBractBaseContinuous;
import com.sw.mes.yb.report.dto.BrokenBractBaseContinuousDto;
import com.sw.mes.yb.report.service.BrokenBractBaseContinuousService;


/**
 * @Author
 * @Description：
 * @Date: Create in
 * @Modified By:
 */
@RestController
@RequestMapping("/BrokenBractBaseContinuous")
@Slf4j
@Api(description = "连续断线接口")
public class BrokenBractBaseContinuousController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private BrokenBractBaseContinuousService brokenBractBaseContinuousService;

	@ApiOperation(value = "分页查询连续断线列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<BrokenBractBaseContinuousDto>> listByPage(ServletRequest request,@ApiIgnore RequestPage requestPage) {
		Specification<BrokenBractBaseContinuous> specification = DynamicSpecifications.bySearchFilter(request, BrokenBractBaseContinuous.class);
    	Page<BrokenBractBaseContinuous> page = brokenBractBaseContinuousService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<BrokenBractBaseContinuousDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有连续断线")
	@GetMapping("/listAll")
	public Result<List<BrokenBractBaseContinuousDto>> listAll(ServletRequest request) {
		Specification<BrokenBractBaseContinuous> specification = DynamicSpecifications.bySearchFilter(request, BrokenBractBaseContinuous.class);
		List<BrokenBractBaseContinuous> brokenBractBaseContinuouss = brokenBractBaseContinuousService.findByExampleSort(specification, null);
		List<BrokenBractBaseContinuousDto> brokenBractBaseContinuousDtos = brokenBractBaseContinuouss.stream()
					.map(brokenBractBaseContinuous -> toLazyDto(brokenBractBaseContinuous))
					.collect(Collectors.toList());
		return ResultGenerator.success(brokenBractBaseContinuousDtos);
	}

	@ApiOperation(value = "连续断线详细信息")
	@GetMapping("/{id}")
	public Result<BrokenBractBaseContinuousDto> get(@PathVariable @ApiParam(value="连续断线id") Long id) {
		BrokenBractBaseContinuous brokenBractBaseContinuous = brokenBractBaseContinuousService.get(id);
		if (null == brokenBractBaseContinuous){
			return ResultGenerator.fail("该连续断线不存在");
		}
		return ResultGenerator.success(toEagerDto(brokenBractBaseContinuous));
	}

	@ApiOperation(value = "创建连续断线")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "连续断线对象") BrokenBractBaseContinuousDto brokenBractBaseContinuousDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		BrokenBractBaseContinuous brokenBractBaseContinuous = fromDto(brokenBractBaseContinuousDto);
		brokenBractBaseContinuousService.saveOrUpdate(brokenBractBaseContinuous);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "更新连续断线")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "连续断线") BrokenBractBaseContinuousDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors()){
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
        if (dto.getId() == null){
            return ResultGenerator.fail("id不能为空");
		}
		BrokenBractBaseContinuous entity = brokenBractBaseContinuousService.get(dto.getId());
        if (entity == null){
            return ResultGenerator.fail("原信息不存在");
		}
        try {
            BrokenBractBaseContinuous entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			brokenBractBaseContinuousService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除连续断线")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
		for (Integer id : ids) {
				brokenBractBaseContinuousService.delete(id);
		}
		return ResultGenerator.success();
	}

	public BrokenBractBaseContinuousDto toLazyDto(BrokenBractBaseContinuous entity) {
		return modelMapper.map(entity, BrokenBractBaseContinuousDto.class);
	}

	public BrokenBractBaseContinuousDto toEagerDto(BrokenBractBaseContinuous entity) {
		BrokenBractBaseContinuousDto dto = modelMapper.map(entity, BrokenBractBaseContinuousDto.class);
		return dto;
	}

	public BrokenBractBaseContinuous fromDto(BrokenBractBaseContinuousDto dto) {
		return modelMapper.map(dto, BrokenBractBaseContinuous.class);
	}


}
