package com.sw.mes.yb.report.controller;

import com.sw.mes.core.context.BaseContextHandler;
import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;
import com.sw.mes.yb.report.entity.ShoulderSurvivalRate;
import com.sw.mes.yb.report.dto.ShoulderSurvivalRateDto;
import com.sw.mes.yb.report.service.ShoulderSurvivalRateService;


/**
* @Author
* @Description：
* @Date: Create in
* @Modified By:
*/
@RestController
@RequestMapping("/shoulderSurvivalRate")
@Slf4j
@Api(description = "ShoulderSurvivalRate接口")
public class ShoulderSurvivalRateController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private ShoulderSurvivalRateService shoulderSurvivalRateService;

	@ApiOperation(value = "分页查询ShoulderSurvivalRate列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<ShoulderSurvivalRateDto>> listByPage(ServletRequest request,@ApiIgnore RequestPage requestPage) {
		Specification<ShoulderSurvivalRate> specification = DynamicSpecifications.bySearchFilter(request, ShoulderSurvivalRate.class);
    	Page<ShoulderSurvivalRate> page = shoulderSurvivalRateService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<ShoulderSurvivalRateDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有ShoulderSurvivalRate")
	@GetMapping("/listAll")
	public Result<List<ShoulderSurvivalRateDto>> listAll(ServletRequest request) {
		Specification<ShoulderSurvivalRate> specification = DynamicSpecifications.bySearchFilter(request, ShoulderSurvivalRate.class);
		List<ShoulderSurvivalRate> shoulderSurvivalRates = shoulderSurvivalRateService.findByExampleSort(specification, null);
		List<ShoulderSurvivalRateDto> shoulderSurvivalRateDtos = shoulderSurvivalRates.stream()
					.map(shoulderSurvivalRate -> toLazyDto(shoulderSurvivalRate))
					.collect(Collectors.toList());
		return ResultGenerator.success(shoulderSurvivalRateDtos);
	}

	@ApiOperation(value = "ShoulderSurvivalRate详细信息")
	@GetMapping("/{id}")
	public Result<ShoulderSurvivalRateDto> get(@PathVariable @ApiParam(value="ShoulderSurvivalRateid") Long id) {
		ShoulderSurvivalRate shoulderSurvivalRate = shoulderSurvivalRateService.get(id);
		if (null == shoulderSurvivalRate)
			return ResultGenerator.fail("该ShoulderSurvivalRate不存在");
		return ResultGenerator.success(toEagerDto(shoulderSurvivalRate));
	}

	@ApiOperation(value = "创建ShoulderSurvivalRate")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "ShoulderSurvivalRate对象") ShoulderSurvivalRateDto shoulderSurvivalRateDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		ShoulderSurvivalRate shoulderSurvivalRate = fromDto(shoulderSurvivalRateDto);
		shoulderSurvivalRateService.saveOrUpdate(shoulderSurvivalRate);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "更新ShoulderSurvivalRate")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "ShoulderSurvivalRate") ShoulderSurvivalRateDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors())
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        if (dto.getId() == null)
            return ResultGenerator.fail("id不能为空");
            ShoulderSurvivalRate entity = shoulderSurvivalRateService.get(dto.getId());
        if (entity == null)
            return ResultGenerator.fail("原信息不存在");
        try {
            ShoulderSurvivalRate entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			shoulderSurvivalRateService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除ShoulderSurvivalRate")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
		for (Integer id : ids) {
				shoulderSurvivalRateService.delete(id);
		}
		return ResultGenerator.success();
	}

	public ShoulderSurvivalRateDto toLazyDto(ShoulderSurvivalRate entity) {
		return modelMapper.map(entity, ShoulderSurvivalRateDto.class);
	}

	public ShoulderSurvivalRateDto toEagerDto(ShoulderSurvivalRate entity) {
		ShoulderSurvivalRateDto dto = modelMapper.map(entity, ShoulderSurvivalRateDto.class);
		return dto;
	}

	public ShoulderSurvivalRate fromDto(ShoulderSurvivalRateDto dto) {
		return modelMapper.map(dto, ShoulderSurvivalRate.class);
	}


}
