package com.sw.mes.yb.turnstore.controller;

import com.sw.mes.core.context.BaseContextHandler;
import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;
import com.sw.mes.yb.turnstore.entity.MaterialWarehousingBinding;
import com.sw.mes.yb.turnstore.dto.MaterialWarehousingBindingDto;
import com.sw.mes.yb.turnstore.service.MaterialWarehousingBindingService;


/**
* @Author
* @Description：
* @Date: Create in
* @Modified By:
*/
@RestController
@RequestMapping("/materialWarehousingBinding")
@Slf4j
@Api(description = "原料库位绑定接口")
public class MaterialWarehousingBindingController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private MaterialWarehousingBindingService materialWarehousingBindingService;

	@ApiOperation(value = "分页查询原料库位绑定列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<MaterialWarehousingBindingDto>> listByPage(ServletRequest request,@ApiIgnore RequestPage requestPage) {
		Specification<MaterialWarehousingBinding> specification = DynamicSpecifications.bySearchFilter(request, MaterialWarehousingBinding.class);
    	Page<MaterialWarehousingBinding> page = materialWarehousingBindingService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<MaterialWarehousingBindingDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有原料库位绑定")
	@GetMapping("/listAll")
	public Result<List<MaterialWarehousingBindingDto>> listAll(ServletRequest request) {
		Specification<MaterialWarehousingBinding> specification = DynamicSpecifications.bySearchFilter(request, MaterialWarehousingBinding.class);
		List<MaterialWarehousingBinding> materialWarehousingBindings = materialWarehousingBindingService.findByExampleSort(specification, null);
		List<MaterialWarehousingBindingDto> materialWarehousingBindingDtos = materialWarehousingBindings.stream()
					.map(materialWarehousingBinding -> toLazyDto(materialWarehousingBinding))
					.collect(Collectors.toList());
		return ResultGenerator.success(materialWarehousingBindingDtos);
	}

	@ApiOperation(value = "原料库位绑定详细信息")
	@GetMapping("/{id}")
	public Result<MaterialWarehousingBindingDto> get(@PathVariable @ApiParam(value="原料库位绑定id") Long id) {
		MaterialWarehousingBinding materialWarehousingBinding = materialWarehousingBindingService.get(id);
		if (null == materialWarehousingBinding)
			return ResultGenerator.fail("该原料库位绑定不存在");
		return ResultGenerator.success(toEagerDto(materialWarehousingBinding));
	}

	@ApiOperation(value = "创建原料库位绑定")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "原料库位绑定对象") MaterialWarehousingBindingDto materialWarehousingBindingDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		MaterialWarehousingBinding materialWarehousingBinding = fromDto(materialWarehousingBindingDto);
		materialWarehousingBindingService.saveOrUpdate(materialWarehousingBinding);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "更新原料库位绑定")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "原料库位绑定") MaterialWarehousingBindingDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors())
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        if (dto.getId() == null)
            return ResultGenerator.fail("id不能为空");
            MaterialWarehousingBinding entity = materialWarehousingBindingService.get(dto.getId());
        if (entity == null)
            return ResultGenerator.fail("原信息不存在");
        try {
            MaterialWarehousingBinding entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			materialWarehousingBindingService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除原料库位绑定")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Long[] ids) {
		for (Long id : ids) {
				materialWarehousingBindingService.delete(id);
		}
		return ResultGenerator.success();
	}

	public MaterialWarehousingBindingDto toLazyDto(MaterialWarehousingBinding entity) {
		return modelMapper.map(entity, MaterialWarehousingBindingDto.class);
	}

	public MaterialWarehousingBindingDto toEagerDto(MaterialWarehousingBinding entity) {
		MaterialWarehousingBindingDto dto = modelMapper.map(entity, MaterialWarehousingBindingDto.class);
		return dto;
	}

	public MaterialWarehousingBinding fromDto(MaterialWarehousingBindingDto dto) {
		return modelMapper.map(dto, MaterialWarehousingBinding.class);
	}


}
