package com.sw.mes.yb.turnstore.controller;

import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import com.sw.mes.yb.turnstore.entity.ProductionBindingTrayLog;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;
import com.sw.mes.core.dto.mes.pc.yb.ProductionBindingTrayLogDto;
import com.sw.mes.yb.turnstore.service.ProductionBindingTrayLogService;


/**
* @Author
* @Description：
* @Date: Create in
* @Modified By:
*/
@RestController
@RequestMapping("/productionBindingTrayLog")
@Slf4j
@Api(description = "托盘绑定/解绑日志记录接口")
public class ProductionBindingTrayLogController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private ProductionBindingTrayLogService productionBindingTrayLogService;

	@ApiOperation(value = "分页查询托盘绑定/解绑日志记录列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<ProductionBindingTrayLogDto>> listByPage(ServletRequest request,@ApiIgnore RequestPage requestPage) {
		Specification<ProductionBindingTrayLog> specification = DynamicSpecifications.bySearchFilter(request, ProductionBindingTrayLog.class);
    	Page<ProductionBindingTrayLog> page = productionBindingTrayLogService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<ProductionBindingTrayLogDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有托盘绑定/解绑日志记录")
	@GetMapping("/listAll")
	public Result<List<ProductionBindingTrayLogDto>> listAll(ServletRequest request) {
		Specification<ProductionBindingTrayLog> specification = DynamicSpecifications.bySearchFilter(request, ProductionBindingTrayLog.class);
		List<ProductionBindingTrayLog> productionBindingTrayLogs = productionBindingTrayLogService.findByExampleSort(specification, null);
		List<ProductionBindingTrayLogDto> productionBindingTrayLogDtos = productionBindingTrayLogs.stream()
					.map(productionBindingTrayLog -> toLazyDto(productionBindingTrayLog))
					.collect(Collectors.toList());
		return ResultGenerator.success(productionBindingTrayLogDtos);
	}

	@ApiOperation(value = "托盘绑定/解绑日志记录详细信息")
	@GetMapping("/{id}")
	public Result<ProductionBindingTrayLogDto> get(@PathVariable @ApiParam(value="托盘绑定/解绑日志记录id") Long id) {
		ProductionBindingTrayLog productionBindingTrayLog = productionBindingTrayLogService.get(id);
		if (null == productionBindingTrayLog) {
            return ResultGenerator.fail("该托盘绑定/解绑日志记录不存在");
        }
		return ResultGenerator.success(toEagerDto(productionBindingTrayLog));
	}

	@ApiOperation(value = "创建托盘绑定/解绑日志记录")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "托盘绑定/解绑日志记录对象") ProductionBindingTrayLogDto productionBindingTrayLogDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		ProductionBindingTrayLog productionBindingTrayLog = fromDto(productionBindingTrayLogDto);
		productionBindingTrayLogService.saveOrUpdate(productionBindingTrayLog);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "更新托盘绑定/解绑日志记录")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "托盘绑定/解绑日志记录") ProductionBindingTrayLogDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        }
        if (dto.getId() == null) {
            return ResultGenerator.fail("id不能为空");
        }
            ProductionBindingTrayLog entity = productionBindingTrayLogService.get(dto.getId());
        if (entity == null) {
            return ResultGenerator.fail("原信息不存在");
        }
        try {
            ProductionBindingTrayLog entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			productionBindingTrayLogService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除托盘绑定/解绑日志记录")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Long[] ids) {
		for (Long id : ids) {
				productionBindingTrayLogService.delete(id);
		}
		return ResultGenerator.success();
	}

	public ProductionBindingTrayLogDto toLazyDto(ProductionBindingTrayLog entity) {
		return modelMapper.map(entity, ProductionBindingTrayLogDto.class);
	}

	public ProductionBindingTrayLogDto toEagerDto(ProductionBindingTrayLog entity) {
		ProductionBindingTrayLogDto dto = modelMapper.map(entity, ProductionBindingTrayLogDto.class);
		return dto;
	}

	public ProductionBindingTrayLog fromDto(ProductionBindingTrayLogDto dto) {
		return modelMapper.map(dto, ProductionBindingTrayLog.class);
	}


}
